# ----------------------------------------------------------------------------
# Copyright (c) 2013--, emperor development team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file LICENSE.md, distributed with this software.
# ----------------------------------------------------------------------------
from __future__ import division
from numpy import argsort, array

from emperor.util import EmperorUnsupportedComputation
from emperor.sort import sort_taxa_table_by_pcoa_coords
from emperor.qiime_backports.biplots import (get_taxa_prevalence,
                                             get_taxa_coords,
                                             make_biplot_scores_output)


def extract_taxa_data(otu_coords, otu_table, lineages, prevalence, N=0):
    """Extract the N most prevalent elements according to a prevalence vector

    Parameters
    ----------
    otu_coords : array_like
         coordinates where specific taxa is centered
    otu_table : array_like
         contingency table
    lineages : list, str
         taxonomic assignments for each row in the otu_table
    prevalence : list, float
         vector with prevalnce from 0 to 1 for each row of the otu_table
         as returned from qiime.biplots.get_taxa_prevalence
    N : int, optional
         number of most prevalent elements to retain, if zero is passed, will
         retain all available

    Returns
    -------
    otu_coords : np.ndarray
         N most prevalent coords
    out_otu_table : np.ndarray
         N most prevalent rows of the otu_table
    out_otu_lineages : np.ndarray
         N most prevalent taxonomic assignments for each row of the
         otu_table
    out_prevalence : np.ndarray
         first N values of prevalence

    Notes
    -----
    Based on qiime.biplots.remove_rare_taxa; though this function opperates on
    generic data that's not in dict forma and returns the appropriate result.
    """
    # If less than zero or greater than length of taxa, N = fix to max
    if N <= 0 or N > len(prevalence):
        N = len(prevalence)

    # get the first N indices to keep from all of the taxa data
    indices = argsort(prevalence)
    indices = indices[::-1][:N]

    # remove the indices that are not needed and return them individually
    out_otu_coords = otu_coords[indices, :]
    out_otu_table = otu_table[indices, :]
    out_otu_lineages = [lineages[index] for index in indices]
    out_prevalence = prevalence[indices]

    return out_otu_coords, out_otu_table, out_otu_lineages, out_prevalence


def preprocess_otu_table(otu_sample_ids, otu_table, lineages,
                         coords_data, coords_headers, N=0):
    """Preprocess the OTU table to to generate the required data for the biplots

    Parameters
    ----------
    otu_sample_ids : array_like
         sample identifiers for the otu_table
    otu_table : array_like
         contingency table
    lineages : list, str
         taxonomic assignments for the OTUs in the otu_table
    coords_data : array_like
         principal coordinates data where the taxa will be mapped
    coords_data : array_like
         principal coordinates data where the taxa will be mapped
    N : int, optional
         number of most prevalent taxa to keep, by default will use all

    Returns
    -------
    otu_coords : np.ndarray
         coordinates representing the N most prevalent taxa in otu_table
    otu_table : np.ndarray
         N most prevalent OTUs from the input otu_table
    otu_lineages : np.ndarray
         taxonomic assignments corresponding to the N most prevalent
         OTUs
    otu_prevalence : np.ndarray
         vector with the prevalence scores of the N highest values
    lines : np.ndarray
         coords where the N most prevalent taxa will be positioned in the
         biplot

    Raises
    ------
    EmperorUnsupportedComputation
        If the contingency table contains one or fewer rows.
    """

    # return empty values if any of the taxa data is empty
    if (otu_sample_ids == []) or (otu_table == array([])) or (lineages == []):
        return [], [], [], [], ''

    # this means there's only one or fewer rows in the contingency table
    if len(otu_table) <= 1 or len(lineages) <= 1:
        raise EmperorUnsupportedComputation(
            "Biplots are not supported for "
            "contingency tables with one or fewer rows")

    # if this element is a list take the first headers and coordinates
    # both of these will be the master coordinates,
    # i. e. where data is centered
    if type(coords_data) == list and type(coords_headers) == list:
        coords_data = coords_data[0]
        coords_headers = coords_headers[0]

    # re-arrange the otu table so it matches the
    # order of the samples in the coordinates
    # data & remove any sample that is not in the
    # coordinates header
    otu_sample_ids, otu_table = sort_taxa_table_by_pcoa_coords(
        coords_headers,
        otu_table, otu_sample_ids)

    # retrieve the prevalence and the coords prior the filtering
    prevalence = get_taxa_prevalence(otu_table)
    bi_plot_coords = get_taxa_coords(otu_table, coords_data)

    o_otu_coords, o_otu_table, o_otu_lineages, o_prevalence =\
        extract_taxa_data(bi_plot_coords, otu_table, lineages, prevalence, N)

    lines = '\n'.join(make_biplot_scores_output(
        {'coord': o_otu_coords, 'lineages': o_otu_lineages}))

    return o_otu_coords, o_otu_table, o_otu_lineages, o_prevalence, lines



# ----------------------------------------------------------------------------
# Copyright (c) 2013--, emperor development team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file COPYING.txt, distributed with this software.
# ----------------------------------------------------------------------------

from __future__ import division


from sys import argv
from os.path import abspath
from datetime import datetime
from StringIO import StringIO
from socket import gethostname

import numpy as np

from emperor.util import (keep_columns_from_mapping_file,
                          get_emperor_library_version)
from emperor._format_strings import (_ELLIPSE_OPACITY_SLIDER,
                                     _VECTORS_OPACITY_SLIDER,
                                     _TAXA_LABELS_SELECTOR,
                                     _TAXA_LABELS_COLOR_SELECTOR,
                                     _BIPLOT_VISIBILITY_SELECTOR,
                                     _BIPLOT_SPHERES_COLOR_SELECTOR,
                                     _EDGES_VISIBILITY_SELECTOR,
                                     _EDGES_COLOR_SELECTOR,
                                     _EMPEROR_FOOTER_HTML_STRING)
from emperor.qiime_backports.format import format_mapping_file
from emperor.qiime_backports.parse import mapping_file_to_dict
from emperor.qiime_backports.filter import (
    sample_ids_from_metadata_description)
from emperor.qiime_backports.util import MetadataMap


class EmperorLogicError(ValueError):
    """Exception raised when a requirement for the Emperor GUI is not met"""
    pass


def format_pcoa_to_js(header, coords, pct_var, custom_axes=[],
                      coords_low=None, coords_high=None, number_of_axes=10,
                      number_of_segments=8):
    """Write the javascript necessary to represent a pcoa file in emperor

    Parameters
    ----------
    header : list of str
        sample names for the pcoa file 1-D array
    coords : array_like
        coordinates of the PCoA file, 2-D array
    pct_var : array_like
        percentage of variation of the PCoA file, 1-D array
    custom_axes : list of str, optional
        list of category names for the custom axes
    coords_low : array_like, optional
        coordinates representing the lower edges of an ellipse
    coords_high : array_like, optional
        coordinates representing the highere edges of an ellipse
    number_of_axes : int, optional
        number of axes to be returned
    number_of_segments : int, optional
        number of segments and rings for each sphere

    Returns
    -------
    str
        javascript representation of the PCoA data inputed, contains a list of
        spheres, list of ellipses (if coords_low and coords_high are present)
        and several setup variables.

    Raises
    ------
    EmperorLogicError
        If the variation explained by at least three axes is not greater than
        0.01.

    Notes
    -----
    Formats the output of qiime_backports.parse.parse_coords_file into
    javascript variable declarations.
    """

    # validating that the number of coords in coords
    if number_of_axes > len(coords[0]):
        number_of_axes = len(coords[0])

    # validating that all the axes are above 0.01%, this accounts for really
    # small variations explained in some axes that end up being not practical
    # as the GUI has some problems when presenting those values on screen
    valid_pcoalabels = len([i for i in pct_var if i > 0.01])
    if number_of_axes > valid_pcoalabels:
        number_of_axes = valid_pcoalabels
    if number_of_axes < 3:
        raise EmperorLogicError("Due to the variation explained, Emperor "
                                "could not plot at least 3 axes, check the "
                                "input files to ensure that the percent "
                                "explained is greater than 0.01 in at least "
                                "three axes.")

    # ranges for the PCoA space
    max_x = np.max(coords[:, 0:1])
    max_y = np.max(coords[:, 1:2])
    max_z = np.max(coords[:, 2:3])
    min_x = np.min(coords[:, 0:1])
    min_y = np.min(coords[:, 1:2])
    min_z = np.min(coords[:, 2:3])
    maximum = np.max(np.abs(coords[:, :number_of_axes]))
    pcoalabels = pct_var[:number_of_axes]

    radius = (max_x-min_x)*.012

    # write the values for all the spheres
    pcoa_str = []
    pcoa_str.append('\nvar g_spherePositions = new Array();\n')
    for point, coord in zip(header, coords):
        pcnts = ["'P%d': %f" % (i+1, coord[i]) for i in range(number_of_axes)]
        all_coords = ', '.join(pcnts)
        pcoa_str.append("g_spherePositions['%s'] = { 'name': '%s', 'color': "
                        "0, 'x': %f, 'y': %f, 'z': %f, %s };\n" %
                        (point, point, coord[0], coord[1], coord[2],
                         all_coords))

    # write the values for all the ellipses
    pcoa_str.append('\nvar g_ellipsesDimensions = new Array();\n')
    if coords_low is not None and coords_high is not None:
        for s_header, s_coord, s_low, s_high in zip(header, coords, coords_low,
                                                    coords_high):
            delta = np.abs(s_high - s_low)
            pcnts = ["'P%d': %f" % (i + 1, s_coord[i]) for i in
                     range(number_of_axes)]
            all_coords = ', '.join(pcnts)

            pcoa_str.append("g_ellipsesDimensions['%s'] = { 'name': '%s', "
                            "'color': 0, 'width': %f, 'height': %f, 'length': "
                            "%f , 'x': %f, 'y': %f, 'z': %f, %s }\n" %
                            (s_header, s_header, delta[0], delta[1], delta[2],
                             s_coord[0], s_coord[1], s_coord[2], all_coords))

    pcoa_str.append('var g_segments = %d, g_rings = %d, g_radius = %f;\n' %
                    (number_of_segments, number_of_segments, radius))
    pcoa_str.append('var g_xAxisLength = %f;\n' % (np.abs(max_x) +
                                                   np.abs(min_x)))
    pcoa_str.append('var g_yAxisLength = %f;\n' % (np.abs(max_y) +
                                                   np.abs(min_y)))
    pcoa_str.append('var g_zAxisLength = %f;\n' % (np.abs(max_z) +
                                                   np.abs(min_z)))
    pcoa_str.append('var g_xMaximumValue = %f;\n' % (max_x))
    pcoa_str.append('var g_yMaximumValue = %f;\n' % (max_y))
    pcoa_str.append('var g_zMaximumValue = %f;\n' % (max_z))
    pcoa_str.append('var g_xMinimumValue = %f;\n' % (min_x))
    pcoa_str.append('var g_yMinimumValue = %f;\n' % (min_y))
    pcoa_str.append('var g_zMinimumValue = %f;\n' % (min_z))
    pcoa_str.append('var g_maximum = %f;\n' % maximum)

    offset = 0

    # create three vars, pc1, pc2 and pc3 if no custom_axes are passed, then
    # use the values of the percent explained by the PCoA; if custom_axes are
    # passed use as many as you can (since customs axes can be either [0, 1, 2,
    # 3])
    for i in range(0, 3):
        try:
            pcoa_str.append('var g_pc%dLabel = \"%s\";\n' % (i + 1,
                                                             custom_axes[i]))
            # offset will help us retrieve the correct pcoalabels val
            offset += 1
        except Exception:
            # if there are custom axes then subtract the number of custom axes
            pcoa_str.append('var g_pc%dLabel = \"PC%d (%.2f %%)\";\n' %
                            (i + 1, i + 1 - offset, pcoalabels[i - offset]))
    pcoa_str.append('var g_number_of_custom_axes = %d;\n' % offset)

    js_pcts = []
    js_pcts_round = []
    if custom_axes is None:
        custom_axes = []
    for element in custom_axes + list(pct_var[:number_of_axes]):
        try:
            # scale the percent so it's a number from 0 to 1
            js_pcts.append('%f' % (float(element)/100))
            js_pcts_round.append('%.2f' % (element))
        except ValueError:
            js_pcts.append('%f' % (float(pct_var[0]/100)))
            js_pcts_round.append('%.2f' % (pct_var[0]))
    pcoa_str.append('var g_fractionExplained = [%s];\n' % ', '.join(js_pcts))
    pcoa_str.append('var g_fractionExplainedRounded = [%s];\n' %
                    ', '.join(js_pcts_round))

    return ''.join(pcoa_str)


def format_mapping_file_to_js(mapping_file_data, mapping_file_headers,
                              columns):
    """Write a javascript representation of the mapping file

    Parameters
    ----------
    mapping_file_data : list of str
        contents of the mapping file
    mapping_file_headers : list of list of str
        headers of the mapping file
    columns : list of str
        valid columns to use, usually a subset of mapping_file_headers

    Returns
    -------
    str
        JavaScript representation of the mapping file
    """
    map_str = []

    mapping_file_dict = mapping_file_to_dict(mapping_file_data,
                                             mapping_file_headers)

    map_values = []
    for k, v in mapping_file_dict.items():
        if 'SampleID' in columns:
            vals = ["'%s'" % k] + ["'%s'" % v[col]
                                   for col in mapping_file_headers[1:]]
        else:
            vals = ["'%s'" % v[col] for col in mapping_file_headers[1:]]
        map_values.append("'%s': [%s]" % (k, ','.join(vals)))

    if 'SampleID' not in columns:
        mapping_file_headers = mapping_file_headers[1:]

    # format the mapping file as javascript objects
    map_str.append('var g_mappingFileHeaders = [%s];\n' %
                   ','.join(["'%s'" % col for col in mapping_file_headers]))
    map_str.append('var g_mappingFileData = { %s };\n' % ','.join(map_values))

    map_object = MetadataMap(mapping_file_dict, [])
    # make sure the comparison for SampleID is made first because otherwise
    # if the metadata map tries to check 'SampleID' it will raise an exception
    animatable_categories = [category for category in columns
                             if category != 'SampleID' and
                             map_object.isNumericCategory(category)]
    map_str.append('var g_animatableMappingFileHeaders = [%s];\n' %
                   ','.join(["'%s'" % col for col in animatable_categories]))

    return ''.join(map_str)


def format_taxa_to_js(otu_coords, lineages, prevalence, min_taxon_radius=0.5,
                      max_taxon_radius=5, radius=1.0):
    """Write a string representing the taxa in a PCoA plot as javascript

    Parameters
    ----------
    otu_coords : array_like
        Numpy array where the taxa is positioned
    lineages : array_like
        Label for each of these lineages
    prevalence : array_like
        Score of prevalence for each of the taxa that is drawn
    min_taxon_radius : float, optional
        Smallest radius for a sphere.
    max_taxon_radius : float, optional
        Largest radius for a spehere.
    radius : float, optional
        Base radius for a sphere.

    Outputs
    -------
    str
        JavaScript string where the taxa information is written to create the
        spheres representing each of these, will return only the variable
        declaration if the inputs are empty.

    Notes
    -----
    These parameters should work more as constants and once we find out that
    there's a value that is too big to be presented, the proper checks should
    be put into place. Currently we haven't found such cases in any study*
    min_taxon_radius: minimum value for the radius of the spheres on the plot
    max_taxon_radious: maximum value for the radius of the spheres on the plot
    radius: default value size
    """
    js_biplots_string = []
    js_biplots_string.append('\nvar g_taxaPositions = new Array();\n')

    # if we have prevalence scores, calculate the taxa radii values
    if len(prevalence):
        taxa_radii = radius * (min_taxon_radius + (max_taxon_radius -
                               min_taxon_radius) * prevalence)
    else:
        taxa_radii = []

    index = 0

    # write the data in the form of a dictionary
    for taxa_label, taxa_coord, t_radius in zip(lineages,
                                                otu_coords, taxa_radii):
        js_biplots_string.append("g_taxaPositions['%d'] = { 'lineage': '%s', "
                                 "'x': %f, 'y': %f, 'z': %f, 'radius': %f};\n"
                                 % (index, taxa_label, taxa_coord[0],
                                    taxa_coord[1], taxa_coord[2], t_radius))
        index += 1
    js_biplots_string.append('\n')

    # join the array of strings as a single string
    return ''.join(js_biplots_string)


def format_vectors_to_js(mapping_file_data, mapping_file_headers, coords_data,
                         coords_headers, connected_by_header,
                         sorted_by_header=None):
    """Write a string representing the vectors in a PCoA plot as javascript

    Parameters
    ----------
    mapping_file_data : list of list of str
        contents of the mapping file
    mapping_file_headers : list of str
        headers of the mapping file
    coords_data : array_like
        coordinates of the PCoA plot in a numpy 2-D array or a list of numpy
        2-D arrays for jackknifed input
    coords_headers : list of str
        headers of the coords in the PCoA plot or a list of lists with the
        headers for jackknifed input
    connected_by_header : list of str
        header of the mapping file that represents how the lines will be
        connected
    sorted_by_header : str, optional
        numeric-only header name to sort the samples in the vectors

    Returns
    -------
    str
        string that represents the vectors in the shape of a javascript object

    Notes
    -----
    If using jackknifed input, the coordinates and headers that will be used
    are the ones belonging to the master coords i. e. the first element.
    """

    vectors_str = []
    vectors_str.append('\nvar g_vectorPositions = new Array();\n')

    if connected_by_header is not None:
        # check if we are processing jackknifed input, if so just get the
        # master
        if type(coords_data) == list:
            coords_data = coords_data[0]
            coords_headers = coords_headers[0]

        columns_to_keep = ['SampleID', connected_by_header]

        # do not ad None if sorted_by_header is None or empty
        if sorted_by_header:
            columns_to_keep.append(sorted_by_header)

        # reduce the amount of data by keeping the required fields only
        mapping_file_data, mapping_file_headers =\
            keep_columns_from_mapping_file(mapping_file_data,
                                           mapping_file_headers,
                                           columns_to_keep)

        # format the mapping file to use this with the filtering function
        mf_string = format_mapping_file(mapping_file_headers,
                                        mapping_file_data)

        index = mapping_file_headers.index(connected_by_header)
        connected_by = list(set([line[index] for line in mapping_file_data]))

        for category in connected_by:
            # convert to StringIO to for each iteration; else the object
            # won't be usable after the first iteration & you'll get an error
            sample_ids = sample_ids_from_metadata_description(StringIO(
                mf_string), '%s:%s' % (connected_by_header, category))

            # if there is a sorting header, sort the coords using these values
            if sorted_by_header:
                sorting_index = mapping_file_headers.index(sorted_by_header)
                to_sort = [line for line in mapping_file_data if line[0] in
                           sample_ids]

                # get the sorted sample ids from the sorted-reduced mapping
                # file
                sample_ids = zip(
                    *sorted(to_sort, key=lambda x: float(x[sorting_index])))[0]

            # each category value is a new vector
            vectors_str.append("g_vectorPositions['%s'] = new Array();\n"
                               % category)

            for s in sample_ids:
                index = coords_headers.index(s)

                # print the first three elements of each coord for each sample
                vectors_str.append("g_vectorPositions['%s']['%s'] = %s;\n"
                                   % (category, s,
                                      coords_data[index, :3].tolist()))

    return ''.join(vectors_str)


def format_comparison_bars_to_js(coords_data, coords_headers, clones,
                                 is_serial_comparison=True):
    """Format coordinates data to create a comparison plot

    Parameters
    ----------
    coords_data : array_like
        numpy array with the replicated coordinates
    coords_headers : list of str
        list with the headers for each of replicated coordinates
    clones : int
        number of replicates in the coords_data and coords_headers
    is_serial_comparison : bool, optional
        whether the samples will be connected one after the other (True) or all
        will originate in the first set of coordinates.

    Returns
    -------
    str
        JavaScript object that contains the data for the comparison plot

    Raises
    ------
    AssertionError
        If the coords_data and coords_headers don't have the same length.
        If the number of clones doesn't concord with the samples being
        presented.

    Notes
    -----
    Unless the value of clones is > 0 this function will return an empty
    javascript object initialization.
    """

    comparison_str = []
    comparison_str.append('\nvar g_comparisonPositions = new Array();\n')

    if is_serial_comparison:
        comparison_str.append('var g_isSerialComparisonPlot = true;\n')
    else:
        comparison_str.append('var g_isSerialComparisonPlot = false;\n')

    if clones:
        headers_length = len(coords_headers)

        # assert some sanity checks
        assert headers_length == len(coords_data), ("The coords data and"
                                                    "the coords headers must "
                                                    "have the same length")
        assert headers_length % clones == 0, ("There has to be an exact number"
                                              "of clones of the data")

        # get the indices that the sample names get sorted by, this will group
        # all the samples with the same prefix together, and since the suffixes
        # are numeric, the samples will be one after the other i. e. sample_0,
        # sample_1, sample_2 and other_0, other_1, other_2 and so on. With
        # these indices sort the coordinates and then the headers themselves,
        # though convert to a numpy array first & back to a list to avoid
        # sorting again
        indices = np.argsort(coords_headers)
        coords_data = coords_data[indices, :]
        coords_headers = np.array(coords_headers)[indices].tolist()

        # in steps of the number of clones iterate through the headers and the
        # coords to create the javascript object with the coordinates
        for index in xrange(0, headers_length, clones):
            # 1st object must have _0 as a suffix, trim it reveal the sample id
            assert coords_headers[index].endswith('_0'), ("There's an internal"
                                                          " inconsistency with"
                                                          " the sample ids")
            sample_id = coords_headers[index][:-2]

            # convert all elements in the numpy array into a string before
            # formatting the elements into the javascript dictionary object
            comparison_str.append("g_comparisonPositions['%s'] = [%s];\n" %
                                  (sample_id, str(', '.join(map(str,
                                   coords_data[index:(index + clones),
                                               0:3].tolist())))))
    return ''.join(comparison_str)


def format_emperor_html_footer_string(has_biplots=False, has_ellipses=False,
                                      has_vectors=False, has_edges=False):
    """Format a footer according to the things being presented in the plot

    This function will remove unnecessary GUI elements from index.html to avoid
    confusions i. e. showing an ellipse opacity slider when there are no
    ellipses in the plot.

    Parameters
    ----------
    has_biplots : bool, optional
        whether the plot has biplots or not
    has_ellipses : bool, optional
        whether the plot has ellipses or not
    has_vectors : bool, optional
        whether the plot has vectors or not
    has_edges : bool, optional
        whether the plot has edges between samples (comparison plot)

    Returns
    -------
    str
        Formatted footer string
    """

    # we use python's built-in ternary operator to add or not a string
    # see _EMPEROR_FOOTER_HTML_STRING
    format_dict = {'biplot_spheres_color_selector':
                   _BIPLOT_SPHERES_COLOR_SELECTOR if has_biplots else '',
                   'biplot_visibility_selector':
                   _BIPLOT_VISIBILITY_SELECTOR if has_biplots else '',
                   'taxa_labels_selector':
                   _TAXA_LABELS_SELECTOR if has_biplots else '',
                   'taxa_labels_color_selector':
                   _TAXA_LABELS_COLOR_SELECTOR if has_biplots else '',
                   'edges_color_selector':
                   _EDGES_COLOR_SELECTOR if has_edges else '',
                   'ellipse_opacity_slider':
                   _ELLIPSE_OPACITY_SLIDER if has_ellipses else '',
                   'vectors_opacity_slider':
                   _VECTORS_OPACITY_SLIDER if has_vectors else '',
                   'edges_visibility_selector':
                   _EDGES_VISIBILITY_SELECTOR if has_edges else ''}

    return _EMPEROR_FOOTER_HTML_STRING.format(**format_dict)


def format_emperor_autograph(metadata_fp, coords_fp, language='HTML'):
    """Create a signature with some meta-data of the Emperor package

    Parameters
    ----------
    metadata_fp : str
        Absolute file path to the metadata mapping file
    coords_fp : str
        Absolute file path to the coordinates file
    language : {'HTML', 'Python', 'C', 'Bash'}, optional
        Language to which it will be formatted as a multi-line comment

    Returns
    -------
    str
        String with information about the executed command

    Raises
    ------
    AssertionError
        If the file is not allowed.
    """

    # supported open and closing of multi-line comments for different languages
    _languages = {'HTML': ('<!--', '-->'), 'Python': ('"""', '"""'),
                  'C': ('/*', '*/'), 'Bash': ('<<COMMENT', 'COMMENT')}

    assert language in _languages.keys(), ('%s is not a supported language' %
                                           language)

    autograph = []
    autograph.append(_languages[language][0])
    autograph.append("*Summary of Emperor's Information*")

    # add the day and time at which the command was called
    autograph.append(datetime.now().strftime('Command executed on %B %d, %Y at'
                                             ' %H:%M:%S'))

    # add library version and SHA-1 if available
    autograph.append('Emperor Version: %s' % get_emperor_library_version())
    autograph.append('HostName: %s' % gethostname())

    # full path to input files
    autograph.append('Metadata: %s' % abspath(metadata_fp))
    autograph.append('Coordinates: %s' % abspath(coords_fp))

    if any([True for element in argv if 'make_emperor.py' in element]):
        autograph.append('Command: %s' % ' '.join(argv))
    else:
        autograph.append('Command: Cannot find direct call to make_emperor.py')
    autograph.append(_languages[language][1])

    return '%s' % '\n'.join(autograph)



# ----------------------------------------------------------------------------
# Copyright (c) 2013--, emperor development team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file LICENSE.md, distributed with this software.
# ----------------------------------------------------------------------------
from __future__ import division

from numpy import zeros
import re


def sort_taxa_table_by_pcoa_coords(coords_header, otu_table, otu_header):
    """Sort and match the samples in the otu table and in the coordinates data

    This function will sort the columns of an otu table as suggested by the
    sample ids in the coords_header

    Parameters
    ----------
    coords_header: list of str
        sample ids that are present in principal coordinates data
    otu_table: numpy array
        numpy array with the data for an otu table
    otu_header: list of str
        sample ids present in the otu table

    Returns
    -------
    sorted_otu_headers: list of str
        sample ids that were present in the coords_header list, the order in
        this table matches the order of the coordinates data
    sorted_otu_table: numpy array
        otu table data with columns belonging to the sample ids in the
        sorted_otu_headers list
    """

    sorted_otu_headers = []

    # the size of the otu table can be pre-allocated for better memory usage
    matching_headers = len(set(coords_header) & set(otu_header))
    sorted_otu_table = zeros([otu_table.shape[0], matching_headers])

    # iterate through the available sample ids in the coordinates file and work
    # only with the ones that are present in the coords and the otu table; the
    # order of the ids is important hence iterate through the original list
    for i, element in enumerate(coords_header):
        if element in otu_header:
            current_index = otu_header.index(element)
            sorted_otu_table[:, i] = otu_table[:, current_index]
            sorted_otu_headers.append(element)

    return sorted_otu_headers, sorted_otu_table


def sort_comparison_filenames(coord_fps):
    """Pass in a list of file names and sort them using the suffix

    Parameters
    ----------
    coord_fps: list of str
        The filenames with the format something_something_qX.txt where X is
        the index of the file.

    Returns
    -------
    list of str
        A sorted version of the list that was passed in where the strings are
        sorted according to the suffix they have, if the string doesn't have a
        suffix it will be added to the beginning of the list.
    """

    if coord_fps == []:
        return []

    def _get_suffix(fp):
        """Gets the number in the suffix for a string using a regex"""
        # any alphanumeric set of characters proceeded by a 'q', a number,
        # a dot & a txt extension at the end of the line. Take for example
        # bray_curtis_q1.txt or unifrac_q11.txt
        regex = re.compile(r'(\w+)_q([0-9]+).txt$')
        tmatch = re.search(regex, fp)

        try:
            number = tmatch.group(2)
        # if the regex doesn't match then put it at the beginning
        except (IndexError, AttributeError):
            number = -1

        return float(number)

    # the key function retrieves the suffix number for the function to sort
    # according to it's floating point representation i. e. the cast to float
    return sorted(coord_fps, key=_get_suffix)



# ----------------------------------------------------------------------------
# Copyright (c) 2013--, emperor development team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file LICENSE.md, distributed with this software.
# ----------------------------------------------------------------------------
__version__ = "0.9.61"  # noqa

from emperor.core import Emperor

__all__ = ['Emperor', 'biplots', 'format', 'filter', 'parse', 'sort', 'util']



#!/usr/bin/env python
# ----------------------------------------------------------------------------
# Copyright (c) 2013--, emperor development team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file LICENSE.md, distributed with this software.
# ----------------------------------------------------------------------------
"""Run all tests.

Originally based on the all_tests.py script from the QIIME project
(http://github.com/qiime/qiime) project at svn revision 3290, now taken from
the E-vident (http://github.com/qiime/evident) project master branch at git SHA
dde2a06f2d990db8b09da65764cd27fc047db788
"""

import re

from os import walk, getcwd, chdir
from sys import exit
from glob import glob
from os.path import join, abspath, dirname, split, exists

from emperor.util import get_emperor_project_dir
from emperor import __version__

from qcli.util import qcli_system_call
from qcli.test import run_script_usage_tests
from qcli.option_parsing import parse_command_line_parameters, make_option

script_info = {}
script_info['brief_description'] = ""
script_info['script_description'] = ""
script_info['script_usage'] = [("", "", "")]
script_info['output_description'] = ""
script_info['required_options'] = []
script_info['optional_options'] = [
    make_option('--suppress_unit_tests', action='store_true', help='suppress '
                " execution of Emperor's unit tests [default: %default]",
                default=False),
    make_option('--suppress_script_usage_tests', action='store_true',
                help="suppress Emperor's script usage tests [default: "
                "%default]", default=False),
    make_option('--suppress_javascript_unit_tests', action='store_true',
                help='suppress Emperor\'s JavaScript unit tests [default: '
                '%default]', default=False),
    make_option('--unittest_glob', help='wildcard pattern to match the unit '
                'tests to execute [default: %default]', default=None),
    make_option('--script_usage_tests', help='comma-separated list of the '
                'script usage tests to execute [default: run all]',
                default=None),
    make_option('-p', '--temp_filepath', type='existing_path', help='temporary'
                ' directory where the script usage tests will be executed',
                default='/tmp/'),
    make_option('--emperor_scripts_dir', help='filepath where the scripts are'
                ' stored', type='existing_path', default=None)
]
script_info['version'] = __version__
script_info['help_on_no_arguments'] = False


def main():
    option_parser, opts, args = parse_command_line_parameters(**script_info)

    unittest_glob = opts.unittest_glob
    temp_filepath = opts.temp_filepath
    script_usage_tests = opts.script_usage_tests
    suppress_unit_tests = opts.suppress_unit_tests
    suppress_script_usage_tests = opts.suppress_script_usage_tests
    suppress_javascript_unit_tests = opts.suppress_javascript_unit_tests

    # since the test data is in the tests folder just add scripts_test_data
    emperor_test_data_dir = join(abspath(dirname(__file__)),
                                 'scripts_test_data/')

    # offer the option for the user to pass the scripts dir from the command
    # line since there is no other way to get the scripts dir. If not provided
    # the base structure of the repository will be assumed. Note that for both
    # cases we are using absolute paths, to avoid unwanted failures.
    if opts.emperor_scripts_dir is None:
        emperor_scripts_dir = abspath(join(get_emperor_project_dir(),
                                           'scripts/'))

        # let's try to guess cases for qiime-deploy type of installs
        if get_emperor_project_dir().endswith('/lib'):
            emperor_scripts_dir = abspath(join(get_emperor_project_dir()[:-3],
                                               'scripts/'))

    else:
        emperor_scripts_dir = abspath(opts.emperor_scripts_dir)

    # make a sanity check
    if (suppress_unit_tests and suppress_script_usage_tests and
            suppress_javascript_unit_tests):
        option_parser.error("All tests have been suppresed. Nothing to run.")

    test_dir = abspath(dirname(__file__))

    unittest_good_pattern = re.compile('OK\s*$')
    application_not_found_pattern = re.compile('ApplicationNotFoundError')
    python_name = 'python'
    bad_tests = []
    missing_application_tests = []

    # Run through all of Emperor's unit tests, and keep track of any files
    # which fail unit tests, note that these are the unit tests only
    if not suppress_unit_tests:
        unittest_names = []
        if not unittest_glob:
            for root, dirs, files in walk(test_dir):
                for name in files:
                    if name.startswith('test_') and name.endswith('.py'):
                        unittest_names.append(join(root, name))
        else:
            for fp in glob(unittest_glob):
                fn = split(fp)[1]
                if fn.startswith('test_') and fn.endswith('.py'):
                    unittest_names.append(abspath(fp))

        unittest_names.sort()

        for unittest_name in unittest_names:
            print "Testing %s:\n" % unittest_name
            command = '%s %s -v' % (python_name, unittest_name)
            stdout, stderr, return_value = qcli_system_call(command)
            print stderr
            if not unittest_good_pattern.search(stderr):
                if application_not_found_pattern.search(stderr):
                    missing_application_tests.append(unittest_name)
                else:
                    bad_tests.append(unittest_name)

    script_usage_failures = 0

    # choose to run some of the script usage tests or all the available ones
    if (not suppress_script_usage_tests and exists(emperor_test_data_dir) and
            exists(emperor_scripts_dir)):
        if script_usage_tests is not None:
            script_tests = script_usage_tests.split(',')
        else:
            script_tests = None

        initial_working_directory = getcwd()

        # Run the script usage testing functionality; note that depending on
        # the module where this was imported, the name of the arguments will
        # change that's the reason why I added the name of the arguments in
        # here
        script_usage_result_summary, script_usage_failures = \
            run_script_usage_tests(emperor_test_data_dir,  # test_data_dir
                                   emperor_scripts_dir,    # scripts_dir
                                   temp_filepath,          # working_dir
                                   True,                   # verbose
                                   script_tests,           # tests
                                   None,                   # failure_log_fp
                                   False)                  # force_overwrite

        # running script usage tests breaks the current working directory
        chdir(initial_working_directory)

    if not suppress_javascript_unit_tests:
        runner = join(test_dir, 'javascript_tests', 'runner.js')
        index = join(test_dir, 'javascript_tests', 'index.html')

        o, e, r = qcli_system_call('phantomjs %s %s' % (runner, index))

        if o:
            print o
        if e:
            print e

        # if all the tests passed
        javascript_tests_passed = True if r == 0 else False
    else:
        javascript_tests_passed = True

    print "==============\nResult summary\n=============="

    if not suppress_unit_tests:
        print "\nUnit test result summary\n------------------------\n"
        if bad_tests:
            print("\nFailed the following unit tests.\n%s"
                  % '\n'.join(bad_tests))

        if missing_application_tests:
            print("\nFailed the following unit tests, in part or whole due "
                  "to missing external applications.\nDepending on the "
                  "Emperor features you plan to use, this may not be "
                  "critical.\n%s" % '\n'.join(missing_application_tests))

        if not(missing_application_tests or bad_tests):
            print "\nAll unit tests passed.\n"

    if not suppress_script_usage_tests:
        if exists(emperor_test_data_dir) and exists(emperor_scripts_dir):
            print("\nScript usage test result summary"
                  "\n--------------------------------\n")
            print script_usage_result_summary
        else:
            print("\nCould not run script usage tests.\nThe Emperor scripts "
                  "directory could not be automatically located, try "
                  "supplying it manually using the --emperor_scripts_dir "
                  "option.")

    if not suppress_javascript_unit_tests:
        print('\nJavaScript unit tests result summary\n'
              '------------------------------------\n')
        if javascript_tests_passed:
            print 'All JavaScript unit tests passed.\n'
        else:
            print 'JavaScript unit tests failed, check the summary above.'

    # In case there were no failures of any type, exit with a return code of 0
    return_code = 1
    if (len(bad_tests) == 0 and len(missing_application_tests) == 0 and
            script_usage_failures == 0 and javascript_tests_passed):
        return_code = 0

    return return_code


if __name__ == "__main__":
    exit(main())



# ----------------------------------------------------------------------------
# Copyright (c) 2013--, emperor development team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file LICENSE.md, distributed with this software.
# ----------------------------------------------------------------------------
from __future__ import division

from numpy import array
from emperor.filter import (filter_samples_from_coords,
                            keep_samples_from_pcoa_data)
from unittest import TestCase, main
from numpy.testing import assert_almost_equal


class TopLevelTests(TestCase):

    def setUp(self):
        self.jk_coords_header = [['1', '2', '3'], ['1', '2', '3'],
                                 ['1', '2', '3'], ['1', '2', '3']]
        self.jk_coords_data = [array([[1.2, 0.1, -1.2], [-2.5, -4.0, 4.5],
                                      [0.11, 5.33, -0.23]]),
                               array([[-1.4, 0.05, 1.3], [2.6, 4.1, -4.7],
                                      [0.14, 2.00, -1.11]]),
                               array([[-1.5, 0.05, 1.6], [2.4, 4.0, -4.8],
                                      [1.0, -0.8, 0.01]]),
                               array([[-1.5, 0.05, 1.6], [2.4, 4.0, -4.8],
                                      [2, 0, 1.11111]])]

        self.coords_header = ['PC.355', 'PC.635', 'PC.636', 'PC.354']
        self.coords_data = COORDS_DATA

    def test_filter_samples_from_coords(self):
        """Check it filters samples from coords data as requested"""
        # check the function raises an exception on an empty set
        self.assertRaises(ValueError, filter_samples_from_coords,
                          self.coords_header, self.coords_data,
                          ['foo', 'bar', 'PC.666'])

        # check it keeps the requested samples
        out_headers, out_coords = \
            filter_samples_from_coords(self.coords_header,
                                       self.coords_data,
                                       ['PC.636', 'PC.355'])
        self.assertEquals(out_headers, ['PC.355', 'PC.636'])
        assert_almost_equal(out_coords, array([[-0.2, -0.1, 0.06, -0.06],
                                               [0.1, -0.1, -0.2, 0.08]]))

        # check it removes the requested samples
        out_headers, out_coords = \
            filter_samples_from_coords(self.coords_header,
                                       self.coords_data,
                                       ['PC.636', 'PC.355'],
                                       negate=True)

        self.assertEquals(out_headers, ['PC.635', 'PC.354'])
        assert_almost_equal(out_coords, array([[-0.3, 0.04, -0.1, 0.15],
                                               [0.04, -0.01, 0.06, -0.34]]))

    def test_remove_samples_from_pcoa_data(self):
        """ """
        # check it keeps the requested samples
        out_headers, out_coords = keep_samples_from_pcoa_data(
            self.coords_header, self.coords_data, ['PC.636', 'PC.355'])
        self.assertEquals(out_headers, ['PC.355', 'PC.636'])
        assert_almost_equal(out_coords, array([[-0.2, -0.1, 0.06, -0.06],
                                               [0.1, -0.1, -0.2, 0.08]]))

        # check it keeps the requested samples when the input is jackknifed
        # data
        out_headers, out_coords = \
            keep_samples_from_pcoa_data(self.jk_coords_header,
                                        self.jk_coords_data,
                                        ['1', '3'])
        self.assertEqual(out_headers,
                         [['1', '3'],
                          ['1', '3'],
                          ['1', '3'],
                          ['1', '3']])

        assert_almost_equal(out_coords, [array([[1.2, 0.1, -1.2],
                                                [0.11, 5.33, -0.23]]),
                                         array([[-1.4, 0.05, 1.3],
                                                [0.14, 2.00, -1.11]]),
                                         array([[-1.5, 0.05, 1.6],
                                                [1.0, -0.8, 0.01]]),
                                         array([[-1.5, 0.05, 1.6],
                                                [2, 0, 1.11111]])])

        # flip the order of the samples to keep
        out_headers, out_coords = keep_samples_from_pcoa_data(
            self.jk_coords_header, self.jk_coords_data, ['3', '1'])
        self.assertEqual(out_headers, [['1', '3'], ['1', '3'],
                                       ['1', '3'], ['1', '3']])
        assert_almost_equal(out_coords, [array([[1.2, 0.1, -1.2],
                                                [0.11, 5.33, -0.23]]),
                                         array([[-1.4, 0.05, 1.3],
                                                [0.14, 2.00, -1.11]]),
                                         array([[-1.5, 0.05, 1.6],
                                                [1.0, -0.8, 0.01]]),
                                         array([[-1.5, 0.05, 1.6],
                                                [2, 0, 1.11111]])])


COORDS_DATA = array([
    [-0.2, -0.1, 0.06, -0.06],
    [-0.3, 0.04, -0.1, 0.15],
    [0.1, -0.1, -0.2, 0.08],
    [0.04, -0.01, 0.06, -0.34]])

if __name__ == "__main__":
    main()



# ----------------------------------------------------------------------------
# Copyright (c) 2013--, emperor development team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file LICENSE.md, distributed with this software.
# ----------------------------------------------------------------------------
from __future__ import division
from numpy import array


def filter_samples_from_coords(headers, coords,
                               valid_sample_ids, negate=False):
    """Filter samples from a pair of headers and coordinates

    Parameters
    ----------
    headers : array_like
         list of sample ids corresponding to the coordinates data
    coords : numpy.ndarray
         numpy array of float values
    valid_sample_ids : list, str
         list of sample ids to keep
    negate: bool, optional
         False means keep the samples in valid_sample_ids,
         True means remove the samples in valid_sample_ids

    Returns
    -------
    out_coord_ids : list, str
         Filtered list headers
    out_coords : list, np.array
         Filtered list of coordinates

    Raises
    ------
    ValueError
        If all the samples are filtered out
    """
    out_coord_ids, out_coords = [], []

    # define the strategy to take for each of the iterations
    if negate:
        def keep_sample(s):
            return s not in valid_sample_ids
    else:
        def keep_sample(s):
            return s in valid_sample_ids

    for sample_id, coord in zip(headers, coords):
        if keep_sample(sample_id):
            out_coord_ids.append(sample_id)
            out_coords.append(array(coord))

    # do not allow empty sets as return values, raise an exception
    if len(out_coord_ids) < 1:
        raise ValueError("All samples have been filtered out")

    return out_coord_ids, array(out_coords)


def keep_samples_from_pcoa_data(headers, coords, sample_ids):
    """Controller function to filter coordinates data according to a list

    Parameters
    ----------
    headers : list, str
        list of sample identifiers, if used for jackknifed data, this
        should be a list of lists containing the sample identifiers
    coords : numpy.ndarray
        2-D numpy array with the float data in the coordinates, if used for
        jackknifed data, coords should be a list of 2-D numpy arrays
    sample_ids : list, str
        list of sample ids that should be kept

    Returns
    -------
    out_headers : list, str
        list of headers
    out_coords : list, np.array
        list of coordinates
    """

    # if the coords are a list then it means that the input jackknifed
    if type(coords) == list:
        out_coords, out_headers = [], []

        for single_headers, single_coords in zip(headers, coords):
            a, b = filter_samples_from_coords(single_headers, single_coords,
                                              sample_ids)

            out_headers.append(a)
            out_coords.append(b)

        return out_headers, out_coords
    else:
        out_headers, out_coords = filter_samples_from_coords(headers,
                                                             coords,
                                                             sample_ids)
        return out_headers, out_coords



# ----------------------------------------------------------------------------
# Copyright (c) 2013--, emperor development team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file LICENSE.md, distributed with this software.
# ----------------------------------------------------------------------------
from __future__ import division

from unittest import TestCase, main
from StringIO import StringIO
from skbio.stats.ordination import OrdinationResults

from emperor.core import Emperor
from _test_core_strings import PCOA_STRING, HTML_STRING


class TopLevelTests(TestCase):
    def setUp(self):
        or_f = StringIO(PCOA_STRING)
        self.ord_res = OrdinationResults.read(or_f)

        self.data = \
            [['PC.354', 'Control', '20061218', 'Ctrol_mouse_I.D._354'],
             ['PC.355', 'Control', '20061218', 'Control_mouse_I.D._355'],
             ['PC.356', 'Control', '20061126', 'Control_mouse_I.D._356'],
             ['PC.481', 'Control', '20070314', 'Control_mouse_I.D._481'],
             ['PC.593', 'Control', '20071210', 'Control_mouse_I.D._593'],
             ['PC.607', 'Fast', '20071112', 'Fasting_mouse_I.D._607'],
             ['PC.634', 'Fast', '20080116', 'Fasting_mouse_I.D._634'],
             ['PC.635', 'Fast', '20080116', 'Fasting_mouse_I.D._635'],
             ['PC.636', 'Fast', '20080116', 'Fasting_mouse_I.D._636']]
        self.headers = ['SampleID', 'Treatment', 'DOB', 'Description']

    def test_str(self):
        emp = Emperor(self.ord_res, self.data, self.headers)
        self.assertItemsEqual(HTML_STRING.split('\n'), str(emp).split('\n'))
        self.assertEqual(HTML_STRING, str(emp))

    def test_ids(self):
        emp = Emperor(self.ord_res, self.data, self.headers)
        self.assertEqual(['PC.354', 'PC.355', 'PC.356', 'PC.481', 'PC.593',
                          'PC.607', 'PC.634', 'PC.635', 'PC.636'],
                         emp.ids)


if __name__ == "__main__":
    main()



# ----------------------------------------------------------------------------
# Copyright (c) 2013--, emperor development team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file LICENSE.md, distributed with this software.
# ----------------------------------------------------------------------------
from __future__ import division

from numpy import ndarray, ones, zeros, vstack
from string import strip

from os import makedirs, listdir
from os.path import abspath, dirname, join, exists, isdir
from copy import deepcopy

from qcli.util import qcli_system_call

from emperor.qiime_backports.make_3d_plots import (get_custom_coords,
                                                   remove_nans,
                                                   scale_custom_coords)
from emperor.qiime_backports.parse import mapping_file_to_dict
from emperor.qiime_backports.util import (MetadataMap, is_valid_git_refname,
                                          is_valid_git_sha1, summarize_pcoas)

from emperor import __version__ as emperor_library_version


class EmperorSupportFilesError(IOError):
    """Exception for missing support files"""
    pass


class EmperorInputFilesError(IOError):
    """Exception for missing support files"""
    pass


class EmperorUnsupportedComputation(ValueError):
    """Exception for computations that lack a meaning"""
    pass


# Based on qiime/qiime/util.py
def get_emperor_library_version():
    """Get Emperor version and the git SHA + current branch (if applicable)"""
    emperor_dir = get_emperor_project_dir()
    emperor_version = emperor_library_version

    # more information could be retrieved following this pattern
    sha_cmd = 'git --git-dir %s/.git rev-parse HEAD' % (emperor_dir)
    sha_o, sha_e, sha_r = qcli_system_call(sha_cmd)
    git_sha = sha_o.strip()

    branch_cmd = 'git --git-dir %s/.git rev-parse --abbrev-ref HEAD' %\
        (emperor_dir)
    branch_o, branch_e, branch_r = qcli_system_call(branch_cmd)
    git_branch = branch_o.strip()

    # validate the output from both command calls
    if is_valid_git_refname(git_branch) and is_valid_git_sha1(git_sha):
        return '%s, %s@%s' % (emperor_version, git_branch, git_sha[0:7])
    else:
        return '%s' % emperor_version


def get_emperor_project_dir():
    """ Returns the top-level Emperor directory

    based on qiime.util.get_qiime_project_dir from github.com/qiime/qiime
    """
    # Get the full path of util.py
    current_file_path = abspath(__file__)
    # Get the directory containing util.py
    current_dir_path = dirname(current_file_path)
    # Return the directory containing the directory containing util.py
    return dirname(current_dir_path)


def get_emperor_support_files_dir():
    """Returns the path for the support files of the project """
    return join(get_emperor_project_dir(), 'emperor/support_files/')


def copy_support_files(file_path):
    """Copy the support files to a named destination

    Parameters
    ----------
    file_path: str
        path where you want the support files to be copied to

    Raises
    ------
    EmperorSupportFilesError
        if a problem is found whilst trying to copy the files.
    """
    file_path = join(file_path, 'emperor_required_resources')

    if not exists(file_path):
        makedirs(file_path)

    # shutil.copytree does not provide an easy way to copy the contents of a
    # directory into another existing directory, hence the system call.
    # use double quotes for the paths to escape any invalid chracter(s)/spaces
    cmd = 'cp -R "%s/"* "%s"' % (get_emperor_support_files_dir(),
                                 abspath(file_path))
    cmd_o, cmd_e, cmd_r = qcli_system_call(cmd)

    if cmd_e:
        raise EmperorSupportFilesError(
            "Error found whilst trying to copy the support files:\n{}\n"
            "Could not execute: {}".format(cmd_e, cmd))

    return


def preprocess_mapping_file(data, headers, columns, unique=False, single=False,
                            clones=0):
    """Process a mapping file to expand the data or remove unuseful fields

    Parameters
    ----------
    data: list of list of str
        mapping file data
    headers: list of str
        mapping file headers
    columns: list of str
        headers to keep, if one of these headers includes two ampersands,
        this function will create a new column by merging the delimited columns
    unique: bool, optional
        keep columns where all values are unique. Default: false.
    single: bool, optional
        keep columns where all values are the same. Default: false
    clones: int, optional
        number of times to replicate the metadata. Default: 0

    Returns
    -------
    data: list of list of str
        processed mapping file data
    headers: list of str
        processed mapping file headers
    """

    # The sample ID must always be there, else it's meaningless data
    if 'SampleID' != columns[0]:
        columns = ['SampleID'] + columns

    # process concatenated columns if needed
    merge = []
    for column in columns:
        # the list can contain None so check "if column" before treating as str
        if column and '&&' in column:
            merge.append(column)
    # each element needs several columns to be merged
    for new_column in merge:
        indices = [headers.index(header_name) for header_name in
                   new_column.split('&&')]

        # join all the fields of the metadata that are listed in indices
        for line in data:
            line.append(''.join([line[index] for index in indices]))
        headers.append(new_column)

    # remove all unique or singled valued columns that are not included in
    # the list of categories that should be kept i. e. columns
    if unique or single:
        columns_to_remove = []
        metadata = MetadataMap(mapping_file_to_dict(data, headers), [])

        # the --coloy_by option in the script interface allows the user to
        # specify the categories you want to use in the generated plot, this
        # the default behaviour is to color by all categories that are not
        # unique. If the user specifies a category with with the --color_by
        # option and this category contains a unique values, this category must
        # still be added thus the structure of the next few lines that
        # form the structure for the two different routes. (1) where no value
        # is specified in the CLI (the value of columns will be [None, x1, x2,
        # x3] where x{1,2,3} are categories requested in other CLI options) and
        # (2) where a value is specified in the CLI.
        #
        # TL;DR
        # see https://github.com/biocore/emperor/issues/271
        if None in columns:
            columns = headers[:]
            f_unique = metadata.hasUniqueCategoryValues
            f_single = metadata.hasSingleCategoryValue
        else:
            def f_unique(x):
                return metadata.hasUniqueCategoryValues(x) and x not in columns

            def f_single(x):
                return metadata.hasSingleCategoryValue(x) and x not in columns

        # find columns that have values that are all unique
        if unique:
            for c in headers[1::]:
                if f_unique(c):
                    columns_to_remove.append(c)
        # remove categories where there is only one value
        if single:
            for c in headers[1::]:
                if f_single(c):
                    columns_to_remove.append(c)
        columns_to_remove = list(set(columns_to_remove))

        # remove the single or unique columns
        data, headers = keep_columns_from_mapping_file(data, headers,
                                                       columns_to_remove,
                                                       negate=True)
    else:
        # when a None is contained in columns, we imply we want to use all the
        # available categories in the mapping file, thus just overwrite the
        # value
        if None in columns:
            columns = headers[:]

    # remove anything not specified in the input
    data, headers = keep_columns_from_mapping_file(data, headers, columns)

    # sanitize the mapping file data and headers
    data, headers = sanitize_mapping_file(data, headers)

    # clones mean: replicate the metadata retagging the sample ids with a
    # suffix
    if clones:
        out_data = []
        for index in range(0, clones):
            out_data.extend([[element[0]+'_%d' % index]+element[1::]
                             for element in data])
        data = out_data

    return data, headers


def keep_columns_from_mapping_file(data, headers, columns, negate=False):
    """Select the header names to remove/keep from the mapping file

    Parameters
    ----------
    data: list of list of str
        mapping file data
    headers: list of str
        mapping file headers names
    columns: list of str
        header names to keep/remove, see negate
    negate: bool, optional
        False will _keep_ the listed columns; True will _remove_ them.
        Default: False

    Returns
    -------
    data: list of list of str
        filtered mapping file data
    headers: list of str
        filtered mapping file headers
    """
    data = deepcopy(data)
    headers = deepcopy(headers)

    if negate:
        indices_of_interest = range(0, len(headers))
    else:
        indices_of_interest = []

    # get the indices that you want to keep; either by removing the
    # indices listed (negate is True) or by adding them (negate is False)
    for column in columns:
        try:
            if negate:
                del indices_of_interest[indices_of_interest.index(
                    headers.index(column))]
            else:
                indices_of_interest.append(headers.index(column))
        except ValueError:
            continue

    # keep the elements at the positions indices
    def keep_elements(elements, indices):
        return [element for i, element in enumerate(elements) if i in indices]

    headers = keep_elements(headers, indices_of_interest)
    data = [keep_elements(row, indices_of_interest) for row in data]

    return data, headers


def preprocess_coords_file(coords_header, coords_data, coords_eigenvals,
                           coords_pct, mapping_header, mapping_data,
                           custom_axes=None, jackknifing_method=None,
                           is_comparison=False,
                           pct_variation_below_one=False):
    """Process a PCoA data and handle customizations in the contents

    This controller function handles any customization that has to be done to
    the PCoA data prior to the formatting. Note that the first element in each
    list (coords, headers, eigenvalues & percents) will be considered the
    master set of coordinates.

    Parameters
    ----------
    coords_header: 1d or 2d array of str
        If 1d array of str, the sample identifiers in the PCoA file
        If 2d array of str, the sample identifiers for each coordinate
        file (if jackknifing or comparing plots)
    coords_data: 2d array of float or list of 2d array of float
        If 2d array of float, matrix of coordinates in the PCoA file
        If list of 2d array of float,  with coordinates for each file
        (if jackknifing or comparing plots)
    coords_eigenvals: 1d or 2d array of float
        If 1d array, eigenvalues for the coordinates file
        If 2d array, list of  arrays with the eigenvalues
        (if jackknifing or comparing plots)
    coords_pct: 1d or 2d array of float
        If 1d array, the percent explained by each principal coordinates axis
        If 2d array, a list of lists with numpy arrays (if jackknifing or
        comparing plots)
    mapping_header: list of str
        mapping file headers names
    mapping_data: list of lists of str
        mapping file data
    custom_axes: str, optional
        name of the mapping data fields to add to coords_data. Default: None
    jackknifing_method: {'sdev', 'IRQ', None}, optional
        For more info see qiime.util.summarize_pcoas. Default: None
    is_comparison: bool, optional
        whether or not the inputs should be considered as the ones for a
        comparison plot. Default: false
    pct_variation_below_one: bool, optional
        boolean to allow percet variation of the axes be under one.
        Default: false

    Returns
    -------
    coords_header: list of str
        Sample identifiers in the PCoA file
    coords_data: 2d array of float
        matrix of coordinates in the PCoA file with custom_axes if provided
    coords_eigenvals: array of float
        either the eigenvalues of the input coordinates or the average
        eigenvalues of the multiple coords that were passed in
    coords_pct: array of float
        list of percents explained by each axis as given by the master
        coordinates i. e. the center around where the values revolve
    coords_low: 2d array of float
        coordinates representing the lower edges of an ellipse; None if no
        jackknifing is applied
    coords_high: 2d array of float
        coordinates representing the highere edges of an ellipse; None if no
        jackknifing is applied
    clones: int
        total number of input files

    Raises
    ------
    AssertionError
        if a comparison plot is requested but a list of data is not passed
        as input
    """

    # prevent obscure and obfuscated errors
    if is_comparison:
        assert type(coords_data) == list, ("Cannot process a comparison with "
                                           "the data from a single "
                                           "coordinates file")

    mapping_file = [mapping_header] + mapping_data
    coords_file = [coords_header, coords_data]

    # number PCoA files; zero for any case except for comparison plots
    clones = 0

    if custom_axes and type(coords_data) == ndarray:
            # sequence ported from qiime/scripts/make_3d_plots.py @ 9115351
            get_custom_coords(custom_axes, mapping_file, coords_file)
            remove_nans(coords_file)
            scale_custom_coords(custom_axes, coords_file)
    elif type(coords_data) == list and not is_comparison:
        # take the first pcoa file as the master set of coordinates
        master_pcoa = [coords_header[0], coords_data[0],
                       coords_eigenvals[0], coords_pct[0]]

        # support pcoas must be a list of lists where each list contain
        # all the elements that compose a coordinates file
        support_pcoas = [[h, d, e, p] for h, d, e, p in zip(coords_header,
                         coords_data, coords_eigenvals, coords_pct)]

        # do not apply procrustes, at least not for now
        coords_data, coords_low, coords_high, eigenvalues_average,\
            identifiers = summarize_pcoas(master_pcoa, support_pcoas,
                                          method=jackknifing_method,
                                          apply_procrustes=False)

        # custom axes and jackknifing is a tricky thing to do, you only have to
        # add the custom values to the master file which is represented as the
        # coords_data return value. Since there is really no variation in that
        # axis then you have to change the values of coords_high and of
        # coords_low to something really small so that WebGL work properly
        if custom_axes:
            coords_file = [master_pcoa[0], coords_data]
            get_custom_coords(custom_axes, mapping_file, coords_file)
            remove_nans(coords_file)
            scale_custom_coords(custom_axes, coords_file)

            # this opens support for as many custom axes as needed
            axes = len(custom_axes)
            coords_low[:, 0:axes] = zeros([coords_low.shape[0], axes])
            coords_high[:, 0:axes] = ones([coords_high.shape[0], axes])*0.00001
            coords_data = coords_file[1]

        if master_pcoa[3][0] < 1.0 and not pct_variation_below_one:
            master_pcoa[3] = master_pcoa[3]*100

        # return a value containing coords_low and coords_high
        return identifiers, coords_data, eigenvalues_average, master_pcoa[3],\
            coords_low, coords_high, clones
    # comparison plots are processed almost individually
    elif type(coords_data) == list and is_comparison:

        # indicates the number of files that were totally processed so other
        # functions/APIs are aware of how many times to replicate the metadata
        clones = len(coords_data)
        out_headers, out_coords = [], []

        for index in range(0, clones):
            headers_i = coords_header[index]
            coords_i = coords_data[index]

            # tag each header with the the number in which those coords came in
            out_headers.extend([element+'_%d' % index for element in
                                headers_i])

            if index == 0:
                # numpy can only stack things if they have the same shape
                out_coords = coords_i

                # the eigenvalues and percents explained are really the ones
                # belonging to the the first set of coordinates that was passed
                coords_eigenvals = coords_eigenvals[index]
                coords_pct = coords_pct[index]
            else:
                out_coords = vstack((out_coords, coords_i))

        coords_file = [out_headers, out_coords]

        if custom_axes:
            # this condition deals with the fact that in order for the custom
            # axes to be added into the original coordinates, we have to add
            # the suffix for the sample identifiers that the coordinates have
            if clones:
                out_data = []
                for index in range(0, clones):
                    out_data.extend([[element[0]+'_%d' % index]+element[1::]
                                     for element in mapping_data])
                mapping_file = [mapping_header] + out_data

            # sequence ported from qiime/scripts/make_3d_plots.py @ 9115351
            get_custom_coords(custom_axes, mapping_file, coords_file)
            remove_nans(coords_file)
            scale_custom_coords(custom_axes, coords_file)

    if coords_pct[0] < 1.0 and not pct_variation_below_one:
        coords_pct = coords_pct*100

    # if no coords summary is applied, return None in the corresponding values
    # note that the value of clones will be != 0 for a comparison plot
    return coords_file[0], coords_file[1], coords_eigenvals, coords_pct, None,\
        None, clones


def _is_numeric(x):
    """Return true if x is a numeric value, return false else

    Parameters
    ----------
    x : str
        string to test whether something is or not a number

    Returns
    -------
    bool
        whether something is or not a number
    """
    try:
        float(x)
    except Exception:
        return False
    return True


def fill_mapping_field_from_mapping_file(data, headers, values,
                                         criteria=_is_numeric):
    """
    Parameters
    ----------
    data : list of list of str
        mapping file data
    headers : list of str
        mapping file headers
    values : str
        string with the format a format of
        Category:ValueToFill;Category:ValueToFill ...
        or
        Category:ColumnToSearch==ValueWithinTheColumnToSearch=ValueToFill
    criteria : function, optional
        function that takes a value and returns true or false, default is
        to check if the inputed value is numeric or not (_is_numeric).

    Returns
    -------
    data: list of list of str
        Filled in mapping file data

    Raises
    ------
    EmperorInputFilesError
        If a header is not found in the mapping file or it wasn't used for
        processing.
    """
    out_data = deepcopy(data)

    # parsing the input values
    values = map(strip, values.split(';'))
    values_dict = {}
    for v in values:
        colname, vals = map(strip, v.split(':', 1))
        vals = map(strip, vals.split(','))
        assert len(vals) == 1, ("You can only pass 1 replacement value:"
                                " {}".format(vals))
        if colname not in values_dict:
            values_dict[colname] = []
        values_dict[colname].extend(vals)

    for key, v in values_dict.items():
        for value in v:
            # variable that is going to contain the name of the column for
            # multiple subtitutions
            column = None
            # variable to control if the values with in a column exist
            used_column_index = False

            try:
                header_index = headers.index(key)
            except ValueError:
                raise EmperorInputFilesError(
                    "The header {} does not exist in the mapping "
                    "file".format(key))

            # for the special case of multiple entries
            if '==' in value and '=' in value:
                arrow_index = value.index('==')
                equal_index = value.rindex('=')
                assert(arrow_index+2) != equal_index and\
                      (arrow_index+1) != equal_index,\
                    "Not properly formatted: {}".format(value)

                column = value[:arrow_index]
                column_value = value[arrow_index+2:equal_index]
                new_value = value[equal_index+1:]

                try:
                    column_index = headers.index(column)
                except ValueError:
                    raise EmperorInputFilesError(
                        "The header {} does not exist in the mapping "
                        "file".format(column))

            # fill in the data
            fill_the_data = False
            for line in out_data:
                if criteria(line[header_index]) is False:
                    if not column:
                        line[header_index] = value
                        used_column_index = True
                        fill_the_data = True
                    else:
                        if line[column_index] == column_value:
                            line[header_index] = new_value
                            used_column_index = True
                            fill_the_data = True

            if not used_column_index and fill_the_data:
                raise EmperorInputFilesError(
                    "This value '{}' does not exist in '{}' or it wasn't used "
                    "forprocessing".format(column_value, column))

    return out_data


def sanitize_mapping_file(data, headers):
    """Clean the strings in the mapping file for use with javascript

    This function will remove all the ocurrences of characters like ' or ".

    Parameters
    ----------
    data : list of lists of str
        the mapping file data
    headers : list of str
        the mapping file headers

    Returns
    -------
    s_data : list of lists of str
        sanitized version of the input mapping file data
    s_headers : list of str
        sanitized version of the input mapping file headers
    """
    all_lines = [headers] + data
    out_lines = []

    # replace single and double quotes with escaped versions of them
    for line in all_lines:
        out_lines.append([element.replace("'", "").replace('"', '')
                          for element in line])

    return out_lines[1::], out_lines[0]


def guess_coordinates_files(dir_path):
    """Given a directory return the file paths that can contain coordinates

    Parameters
    ----------
    dir_path : str
        path to the directory where coordinate files are contained

    Returns
    -------
    list of str
        list of filepaths pointing to the coordinates files

    Notes
    -----
    If a path inside dir_path meets any of the following criteria, it will be
    ignored:
    - Is a hidden file
    - Is named `Icon?`.
    - Is folder
    - Is part of the procrustes results from QIIME, see
      transform_coordinate_matrices.py
    """
    coord_fps = []

    for filepath in listdir(dir_path):
        if filepath.startswith('.'):
            continue
        if filepath.startswith('Icon?'):
            continue

        # we need the full path for the next check
        filepath = join(abspath(dir_path), filepath)

        if isdir(filepath):
            continue
        if filepath.endswith('procrustes_results.txt'):
            continue

        coord_fps.append(filepath)

    return coord_fps



# ----------------------------------------------------------------------------
# Copyright (c) 2013--, emperor development team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file LICENSE.md, distributed with this software.
# ----------------------------------------------------------------------------
from __future__ import division

from unittest import TestCase, main
from tempfile import mkstemp
from os import close
from StringIO import StringIO

import numpy as np
import numpy.testing as npt

from emperor.parse import parse_coords


class ParseTests(TestCase):

    def test_parse_coords_ordination_results(self):
        """parse_coords should handle skbio's OrdinationResults file"""
        coords = StringIO(ordination_results_file)

        obs = parse_coords(coords)
        exp = (['A', 'B', 'C'],
               np.array([[.11, .09, .23], [.03, .07, -.26], [.12, .06, -.32]]),
               np.array([4.94, 1.79, 1.50]),
               np.array([14.3, 5.2, 4.3]))
        # test the header and the values apart from each other
        self.assertEqual(obs[0], exp[0])
        npt.assert_almost_equal(obs[1], exp[1])
        npt.assert_almost_equal(obs[2], exp[2])
        npt.assert_almost_equal(obs[3], exp[3])

    def test_parse_coords_qiime(self):
        """parse_coords should handle old qiime PCoA coords format"""
        coords = StringIO(qiime_pcoa_file)
        obs = parse_coords(coords)
        exp = (['A', 'B', 'C'],
               np.array([[.11, .09, .23], [.03, .07, -.26], [.12, .06, -.32]]),
               np.array([4.94, 1.79, 1.50]),
               np.array([14.3, 5.2, 4.3]))
        # test the header and the values apart from each other
        self.assertEqual(obs[0], exp[0])
        npt.assert_almost_equal(obs[1], exp[1])
        npt.assert_almost_equal(obs[2], exp[2])
        npt.assert_almost_equal(obs[3], exp[3])

    def test_parse_coords_qiime_file(self):
        """parse_coords should handle old qiime PCoA coords file"""
        fd, fp = mkstemp()
        close(fd)

        with open(fp, 'w') as f:
            f.write(qiime_pcoa_file)

        with open(fp, 'U') as f:
            obs = parse_coords(f)

        exp = (['A', 'B', 'C'],
               np.array([[.11, .09, .23], [.03, .07, -.26], [.12, .06, -.32]]),
               np.array([4.94, 1.79, 1.50]),
               np.array([14.3, 5.2, 4.3]))
        # test the header and the values apart from each other
        self.assertEqual(obs[0], exp[0])
        npt.assert_almost_equal(obs[1], exp[1])
        npt.assert_almost_equal(obs[2], exp[2])
        npt.assert_almost_equal(obs[3], exp[3])


ordination_results_file = u"""Eigvals\t3
4.94\t1.79\t1.50

Proportion explained\t3
14.3\t5.2\t4.3

Species\t0\t0

Site\t3\t3
A\t.11\t.09\t.23
B\t.03\t.07\t-.26
C\t.12\t.06\t-.32

Biplot\t0\t0

Site constraints\t0\t0"""

qiime_pcoa_file = """pc vector number\t1\t2\t3
A\t0.11\t0.09\t0.23
B\t0.03\t0.07\t-0.26
C\t0.12\t0.06\t-0.32


eigvals\t4.94\t1.79\t1.50
% variation explained\t14.3\t5.2\t4.3


"""

if __name__ == '__main__':
    main()



# ----------------------------------------------------------------------------
# Copyright (c) 2013--, emperor development team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file LICENSE.md, distributed with this software.
# ----------------------------------------------------------------------------
from __future__ import division

from numpy import array
from emperor.format import (format_pcoa_to_js, format_mapping_file_to_js,
                            format_taxa_to_js, format_vectors_to_js,
                            format_emperor_html_footer_string,
                            EmperorLogicError, format_comparison_bars_to_js,
                            format_emperor_autograph)
from _test_format_strings import (PCOA_JS, PCOA_JS_CUSTOM_AXES,
                                  PCOA_JS_JACKKNIFED, PCOA_JS_SEGMENTS,
                                  MAPPING_FILE_JS,
                                  MAPPING_FILE_JS_WITH_ANIMATABLE_CATEGORIES,
                                  TAXA_JS_STRING, VECTOR_JS_STRING_NO_SORTING,
                                  VECTOR_JS_STRING_SORTING,
                                  COMPARISON_JS_STRING,
                                  COMPARISON_JS_STRING_NON_SERIAL,
                                  COMPARISON_COORDS_HEADERS_ZERO,
                                  EXPECTED_FOOTER_A, EXPECTED_FOOTER_B,
                                  EXPECTED_FOOTER_C, EXPECTED_FOOTER_D,
                                  EXPECTED_FOOTER_E)
from unittest import TestCase, main


class TopLevelTests(TestCase):

    def setUp(self):
        self.pcoa_pct_var = array([2.66887049e+01, 1.62563704e+01,
                                   1.37754129e+01, 1.12172158e+01,
                                   1.00247750e+01, 8.22835130e+00,
                                   7.55971174e+00, 6.24945796e+00,
                                   1.17437419e-14])
        self.pcoa_pct_var_really_low = array([2.66887049e+01, 1.62563704e+01,
                                              0.001, 0.0001, 0.0019, 0.0018,
                                              0.0017, 0.0016, 0.0015])
        self.pcoa_headers = ['PC.355', 'PC.607', 'PC.634', 'PC.635', 'PC.593',
                             'PC.636', 'PC.481', 'PC.354', 'PC.356']
        self.pcoa_coords = PCOA_DATA
        self.pcoa_eigen_values = array([4.79412119e-01, 2.92014956e-01,
                                        2.47449246e-01, 2.01496072e-01,
                                        1.80076128e-01, 1.47806773e-01,
                                        1.35795927e-01, 1.12259696e-01,
                                        2.10954117e-16])

        # data specific for testing the jackknifing
        self.pcoa_jk_headers = ['PC.355', 'PC.607', 'PC.634', 'PC.635']
        self.pcoa_jk_coords = array([[0.3, 0.5, 0.1, 0.3],
                                     [1.1, 1.1, 1.0, 0.8],
                                     [0.1, 3.3, 5.5, 0.1],
                                     [1.0, 2.0, 1.0, 1.0]])
        self.pcoa_jk_eigen_values = array([0.45, 0.32, 0.21, 0.02])
        self.pcoa_jk_pct_var = array([44, 40, 15, 1])
        self.pcoa_jk_coords_low = array([[0.2, 0.3, 0.1, 0.3],
                                         [1.1, 0.1, 0.0, 0.3],
                                         [0.6, 3.1, 1.5, 0.1],
                                         [0.023, 1.0, 0.01, 1.0]])
        self.pcoa_jk_coords_high = array([[0.6, 0.8, 0.9, 0.31],
                                          [1, 2.1, 0.0, 0.8],
                                          [0.9, 3.7, 5.5, 0.1111],
                                          [0.01222, 2.0, 0.033, 2.0]])

        self.mapping_file_data = MAPPING_FILE_DATA
        self.mapping_file_headers = ['SampleID', 'BarcodeSequence',
                                     'LinkerPrimerSequence', 'Treatment',
                                     'DOB', 'Description']
        self.good_columns = ['Treatment', 'LinkerPrimerSequence']
        self.columns_with_gradients = ['Treatment', 'LinkerPrimerSequence',
                                       'DOB']

        self.otu_coords = array([[2.80399118e-01, -6.01282860e-03,
                                  2.34854344e-02, -4.68109475e-02,
                                  -1.46624450e-01, 5.66979125e-03,
                                  -3.54299634e-02, -2.55785794e-01,
                                  -4.84141987e-09],
                                 [2.28820400e-01, -1.30142097e-01,
                                  -2.87149448e-01, 8.64498846e-02,
                                  4.42951919e-02, 2.06042607e-01,
                                  3.10003571e-02, 7.19920437e-02,
                                  -4.84141987e-09],
                                 [-9.13299284e-02, 4.24147148e-01,
                                  -1.35627421e-01, -5.75194809e-02,
                                  1.51363491e-01, -2.53935676e-02,
                                  5.17306152e-02, -3.87382176e-02,
                                  -4.84141987e-09],
                                 [-2.76542164e-01, -1.44964375e-01,
                                  6.66467344e-02, -6.77109454e-02,
                                  1.76070270e-01, 7.29693901e-02,
                                  -2.29889464e-01, -4.65989417e-02,
                                  -4.84141987e-09]])
        self.lineages = ['Root;k__Bacteria;p__Firmicutes',
                         'Root;k__Bacteria;p__Bacteroidetes',
                         'Root;k__Bacteria;p__Tenericutes',
                         'Root;k__Bacteria;Other']
        self.prevalence = array([1., 0.66471926, 0.08193196, 0.04374296])

        # comparison test
        self.comparison_coords_data = \
            array([[-0.0677, -2.036, 0.2726, 1.051, -0.180, -0.698],
                   [-1.782, -0.972, 0.1582, -1.091, 0.531, 0.292],
                   [-0.659, -0.2566, 0.514, -2.698, -0.393, 0.420],
                   [-1.179, -0.968, 2.525, 0.53, -0.529, 0.632],
                   [-0.896, -1.765, 0.274, -0.3235, 0.4009, -0.03497],
                   [-0.0923, 1.414, -0.622, 0.298, 0.5, -0.4580],
                   [-0.972, 0.551, 1.144, 0.3147, -0.476, -0.4279],
                   [1.438, -2.603, -1.39, 1.300, -0.1606, 1.260],
                   [-0.356, 0.0875, 0.772, 0.539, -0.586, -1.431],
                   [1.512, -1.239, -0.0365, -0.682, -0.971, 0.356],
                   [1.17, 1.31, -1.407, 1.6, 0.60, 2.26],
                   [2.618, 0.739, -0.01295, -0.937, 3.079, -2.534],
                   [0.2339, -0.880, -1.753, 0.177, 0.3517, -0.743],
                   [0.436, 2.12, -0.935, -0.476, -0.805, 0.4164],
                   [-0.880, 1.069, 1.069, -0.596, -0.199, 0.306],
                   [0.294, 0.2988, 0.04670, -0.3865, 0.460, -0.431],
                   [1.640, 0.2485, -0.354, 1.43, 1.226, 1.095],
                   [0.821, -1.13, -1.794, -1.171, -1.27, -0.842]])
        self.comparison_coords_headers = ['sampa_0', 'sampb_0', 'sampc_0',
                                          'sampd_0', 'sampe_0', 'sampf_0',
                                          'sampa_1', 'sampb_1', 'sampc_1',
                                          'sampd_1', 'sampe_1', 'sampf_1',
                                          'sampa_2', 'sampb_2', 'sampc_2',
                                          'sampd_2', 'sampe_2', 'sampf_2']
        self.comparison_coords_headers_zero = ['sampa0_0', 'sampb0_0',
                                               'sampc0_0', 'sampd00_0',
                                               'sampe00_0', 'sampf00_0',
                                               'sampa0_1', 'sampb0_1',
                                               'sampc0_1', 'sampd00_1',
                                               'sampe00_1', 'sampf00_1',
                                               'sampa0_2', 'sampb0_2',
                                               'sampc0_2', 'sampd00_2',
                                               'sampe00_2', 'sampf00_2']

    def test_format_pcoa_to_js(self):
        """Test correct formatting of the PCoA file"""
        # test the case with only points and nothing else
        out_js_pcoa_string = \
            format_pcoa_to_js(self.pcoa_headers, self.pcoa_coords,
                              self.pcoa_pct_var)
        self.assertEquals(out_js_pcoa_string, PCOA_JS)

        # test custom axes and the labels
        out_js_pcoa_string = \
            format_pcoa_to_js(self.pcoa_headers, self.pcoa_coords,
                              self.pcoa_pct_var,
                              custom_axes=['Instant'])
        self.assertEquals(out_js_pcoa_string, PCOA_JS_CUSTOM_AXES)

        # test jackknifed pcoa plots
        out_js_pcoa_string = \
            format_pcoa_to_js(self.pcoa_jk_headers, self.pcoa_jk_coords,
                              self.pcoa_jk_pct_var,
                              coords_low=self.pcoa_jk_coords_low,
                              coords_high=self.pcoa_jk_coords_high)
        self.assertEquals(out_js_pcoa_string, PCOA_JS_JACKKNIFED)

        # check it raises an exception when the variation explained on the
        # axes is not greater than 0.01 for at least three of them
        self.assertRaises(EmperorLogicError, format_pcoa_to_js,
                          self.pcoa_headers, self.pcoa_coords,
                          self.pcoa_pct_var_really_low)

        # test segments
        out_js_pcoa_string = \
            format_pcoa_to_js(self.pcoa_jk_headers, self.pcoa_jk_coords,
                              self.pcoa_jk_pct_var,
                              coords_low=self.pcoa_jk_coords_low,
                              coords_high=self.pcoa_jk_coords_high,
                              number_of_segments=14)
        self.assertEquals(out_js_pcoa_string, PCOA_JS_SEGMENTS)

    def test_format_mapping_file_to_js(self):
        """Tests correct formatting of the metadata mapping file"""

        # regular case, no categories that can be animated
        out_js_mapping_file_string = format_mapping_file_to_js(
            self.mapping_file_data, self.mapping_file_headers,
            self.good_columns)
        self.assertEquals(out_js_mapping_file_string, MAPPING_FILE_JS)

        # case with categories that can be animated i. e. that work as
        # gradients
        out_js_mapping_file_string = format_mapping_file_to_js(
            self.mapping_file_data, self.mapping_file_headers,
            self.columns_with_gradients)
        self.assertEquals(out_js_mapping_file_string,
                          MAPPING_FILE_JS_WITH_ANIMATABLE_CATEGORIES)

    def test_format_taxa_to_js(self):
        """Tests correct formatting of the taxa"""
        out_js_taxa_string = format_taxa_to_js(self.otu_coords, self.lineages,
                                               self.prevalence)
        self.assertEquals(out_js_taxa_string, TAXA_JS_STRING)

        # case with empty data
        out_js_taxa_string = format_taxa_to_js([], [], [])
        self.assertEquals(out_js_taxa_string, "\nvar g_taxaPositions = "
                          "new Array();\n\n")

    def test_format_vectors_to_js(self):
        """Tests correct formatting of the vectors from the coords"""

        # test that only the variable declaration gets returned
        out_js_vector_string = format_vectors_to_js(self.mapping_file_data,
                                                    self.mapping_file_headers,
                                                    self.pcoa_coords,
                                                    self.pcoa_headers, None,
                                                    None)
        self.assertEquals(out_js_vector_string, '\nvar g_vectorPositions = new'
                          ' Array();\n')

        # vector string without sorting for the coordinates
        out_js_vector_string = format_vectors_to_js(self.mapping_file_data,
                                                    self.mapping_file_headers,
                                                    self.pcoa_coords,
                                                    self.pcoa_headers,
                                                    'Treatment', None)
        self.assertEquals(out_js_vector_string, VECTOR_JS_STRING_NO_SORTING)

        # vector string sorting by the DOB category
        out_js_vector_string = format_vectors_to_js(self.mapping_file_data,
                                                    self.mapping_file_headers,
                                                    self.pcoa_coords,
                                                    self.pcoa_headers,
                                                    'Treatment', 'DOB')
        self.assertEquals(out_js_vector_string, VECTOR_JS_STRING_SORTING)

    def test_format_comparison_bars_to_js(self):
        """Check the correct strings are created for the two types of inputs"""

        # empty string generation for comparison i. e. no clones
        out_js_comparison_string = \
            format_comparison_bars_to_js(self.comparison_coords_data,
                                         self.comparison_coords_headers, 0,
                                         True)
        self.assertEquals(out_js_comparison_string, '\nvar '
                          'g_comparisonPositions = new Array();\nvar '
                          'g_isSerialComparisonPlot = true;\n')

        out_js_comparison_string = \
            format_comparison_bars_to_js(self.comparison_coords_data,
                                         self.comparison_coords_headers, 3,
                                         True)
        self.assertEquals(out_js_comparison_string, COMPARISON_JS_STRING)

        # empty string generation for comparison i. e. no clones
        out_js_comparison_string = \
            format_comparison_bars_to_js(self.comparison_coords_data,
                                         self.comparison_coords_headers, 0,
                                         False)
        self.assertEquals(out_js_comparison_string, '\nvar '
                          'g_comparisonPositions = new Array();\nvar '
                          'g_isSerialComparisonPlot = false;\n')

        out_js_comparison_string = format_comparison_bars_to_js(
            self.comparison_coords_data, self.comparison_coords_headers, 3,
            False)
        self.assertEquals(out_js_comparison_string,
                          COMPARISON_JS_STRING_NON_SERIAL)

        out_js_comparison_string = format_comparison_bars_to_js(
            self.comparison_coords_data, self.comparison_coords_headers_zero,
            3, False)
        self.assertEquals(out_js_comparison_string,
                          COMPARISON_COORDS_HEADERS_ZERO)

    def test_format_comparison_bars_to_js_exceptions(self):
        """Check the correct exceptions are raised for incorrect inputs"""

        # assertion for wrong length in headers
        self.assertRaises(AssertionError, format_comparison_bars_to_js, [],
                          self.comparison_coords_data, 3)

        # assertion for wrong length in coords data
        self.assertRaises(AssertionError, format_comparison_bars_to_js,
                          self.comparison_coords_headers,
                          self.comparison_coords_data[1::], 3)

        # assertion for wrong number of clones and elements
        self.assertRaises(AssertionError, format_comparison_bars_to_js,
                          self.comparison_coords_headers,
                          self.comparison_coords_data, 11)

    def test_format_emperor_html_footer_string(self):
        """Test correct formatting of the footer string"""
        self.maxDiff = 5000

        # footer for a jackknifed pcoa plot without biplots
        out_string = format_emperor_html_footer_string(False, True)
        self.assertItemsEqual(out_string.split('\n'),
                              EXPECTED_FOOTER_A.split('\n'))
        self.assertEqual(out_string, EXPECTED_FOOTER_A)

        # footer for biplots without jackknifing
        out_string = format_emperor_html_footer_string(True, False)
        self.assertItemsEqual(out_string.split('\n'),
                              EXPECTED_FOOTER_B.split('\n'))
        self.assertEqual(out_string, EXPECTED_FOOTER_B)

        # no biplots nor jackknifing
        out_string = format_emperor_html_footer_string(False, False)
        self.assertItemsEqual(out_string.split('\n'),
                              EXPECTED_FOOTER_C.split('\n'))
        self.assertEqual(out_string, EXPECTED_FOOTER_C)

        #  no biplots no jackknifing but with vectors
        out_string = format_emperor_html_footer_string(False, False, True)
        self.assertItemsEqual(out_string.split('\n'),
                              EXPECTED_FOOTER_D.split('\n'))
        self.assertEqual(out_string, EXPECTED_FOOTER_D)

        # comparison plot
        out_string = format_emperor_html_footer_string(False, False,
                                                       False, True)
        self.assertItemsEqual(out_string.split('\n'),
                              EXPECTED_FOOTER_E.split('\n'))
        self.assertEqual(out_string, EXPECTED_FOOTER_E)

    def test_format_emperor_autograph(self):
        """Test signatures are created correctly for each of language"""

        autograph = format_emperor_autograph('mapping_file.txt',
                                             'pcoa_unweighted_unifrac.txt')

        # check for comment open and comment close
        self.assertTrue('<!--' in autograph)
        self.assertTrue('-->' in autograph)
        # check for fields since we cannot check for the specifics
        self.assertTrue("*Summary of Emperor's Information*" in autograph)
        self.assertTrue('Metadata:' in autograph)
        self.assertTrue('Coordinates:' in autograph)
        self.assertTrue('HostName:' in autograph)
        self.assertTrue("Command:" in autograph)
        self.assertTrue("Emperor Version: " in autograph)
        self.assertTrue("Command executed on " in autograph)

        autograph = format_emperor_autograph('mapping_file.txt',
                                             'pcoa_unweighted_unifrac.txt',
                                             'Python')
        # check for comment open and comment close
        self.assertTrue('"""' in autograph)
        self.assertTrue('"""' in autograph)
        # check for fields since we cannot check for the specifics
        self.assertTrue("*Summary of Emperor's Information*" in autograph)
        self.assertTrue('Metadata:' in autograph)
        self.assertTrue('Coordinates:' in autograph)
        self.assertTrue('HostName:' in autograph)
        self.assertTrue("Command:" in autograph)
        self.assertTrue("Emperor Version: " in autograph)
        self.assertTrue("Command executed on " in autograph)

        autograph = format_emperor_autograph('mapping_file.txt',
                                             'pcoa_unweighted_unifrac.txt',
                                             'C')
        # check for comment open and comment close
        self.assertTrue('/*' in autograph)
        self.assertTrue('*/' in autograph)
        # check for fields since we cannot check for the specifics
        self.assertTrue("*Summary of Emperor's Information*" in autograph)
        self.assertTrue('Metadata:' in autograph)
        self.assertTrue('Coordinates:' in autograph)
        self.assertTrue('HostName:' in autograph)
        self.assertTrue("Command:" in autograph)
        self.assertTrue("Emperor Version: " in autograph)
        self.assertTrue("Command executed on " in autograph)

        autograph = format_emperor_autograph('mapping_file.txt',
                                             'pcoa_unweighted_unifrac.txt',
                                             'Bash')
        # check for comment open and comment close
        self.assertTrue('<<COMMENT' in autograph)
        self.assertTrue('COMMENT' in autograph)
        # check for fields since we cannot check for the specifics
        self.assertTrue("*Summary of Emperor's Information*" in autograph)
        self.assertTrue('Metadata:' in autograph)
        self.assertTrue('Coordinates:' in autograph)
        self.assertTrue('HostName:' in autograph)
        self.assertTrue("Command:" in autograph)
        self.assertTrue("Emperor Version: " in autograph)
        self.assertTrue("Command executed on " in autograph)

        # haskell and cobol are ... not supported
        self.assertRaises(AssertionError, format_emperor_autograph,
                          'mapping_file.txt', 'pcoa.txt', 'Haskell')
        self.assertRaises(AssertionError, format_emperor_autograph,
                          'mapping_file.txt', 'pcoa.txt', 'Cobol')


PCOA_DATA = array([[-1.09166142e-01, 8.77774496e-02, 1.15866606e-02,
                    -6.26863896e-02, 2.31533068e-02, 8.76934639e-02,
                    1.37400927e-03, -1.35496063e-05, 1.29849404e-09],
                   [6.88959784e-02, -1.66234067e-01, -9.98300962e-02,
                    -2.90522450e-02, 5.05569953e-02, -2.95200038e-03,
                    -3.25863204e-02, -2.17218431e-02, 1.29849404e-09],
                   [2.04684540e-01, 1.28911236e-01, -2.93614192e-02,
                    1.07657904e-01, 1.78480761e-02, 7.97778676e-03,
                    -2.92003235e-02, -1.23468947e-03, 1.29849404e-09],
                   [1.26131510e-01, -2.66030272e-03, -1.41717093e-01,
                    -9.71089687e-03, -6.94272590e-02, 3.67235068e-03,
                    4.29867599e-02, 6.44276242e-03, 1.29849404e-09],
                   [9.68466168e-02, -1.59388265e-01, 1.35271607e-01,
                    5.12015857e-02, -2.02552984e-02, 3.07034843e-02,
                    1.55159338e-02, 1.42426937e-02, 1.29849404e-09],
                   [2.81534642e-01, 7.10660196e-02, 9.71542020e-02,
                    -8.06472757e-02, 7.04245456e-03, -4.53133767e-02,
                    6.55825124e-03, -1.26412251e-02, 1.29849404e-09],
                   [-1.92382819e-01, 1.47832029e-02, -1.47871039e-02,
                    1.90888050e-02, 7.26409669e-02, -3.73008815e-02,
                    3.94304860e-02, 3.25351917e-02, 1.29849404e-09],
                   [-2.93353176e-01, 1.83956004e-02, 3.29884266e-02,
                    3.15360631e-02, -2.86943531e-02, -1.94225139e-02,
                    8.06272805e-03, -5.58094095e-02, 1.29849404e-09],
                   [-1.83191151e-01, 34912621e-03, 8.69481594e-03,
                    -2.73875510e-02, -5.28648893e-02, -2.50583131e-02,
                    -5.21415245e-02, 3.82000689e-02, 1.29849404e-09]])

MAPPING_FILE_DATA = [['PC.354', 'AGCACGAGCCTA', 'YATGCTGCCTCCCGTAGGAGT',
                      'Control', '20061218', 'Control_mouse_I.D._354'],
                     ['PC.355', 'AACTCGTCGATG', 'YATGCTGCCTCCCGTAGGAGT',
                      'Control', '20061218', 'Control_mouse_I.D._355'],
                     ['PC.356', 'ACAGACCACTCA', 'YATGCTGCCTCCCGTAGGAGT',
                      'Control', '20061126', 'Control_mouse_I.D._356'],
                     ['PC.481', 'ACCAGCGACTAG', 'YATGCTGCCTCCCGTAGGAGT',
                      'Control', '20070314', 'Control_mouse_I.D._481'],
                     ['PC.593', 'AGCAGCACTTGT', 'YATGCTGCCTCCCGTAGGAGT',
                      'Control', '20071210', 'Control_mouse_I.D._593'],
                     ['PC.607', 'AACTGTGCGTAC', 'YATGCTGCCTCCCGTAGGAGT',
                      'Fast', '20071112', 'Fasting_mouse_I.D._607'],
                     ['PC.634', 'ACAGAGTCGGCT', 'YATGCTGCCTCCCGTAGGAGT',
                      'Fast', '20080116', 'Fasting_mouse_I.D._634'],
                     ['PC.635', 'ACCGCAGAGTCA', 'YATGCTGCCTCCCGTAGGAGT',
                      'Fast', '20080116', 'Fasting_mouse_I.D._635'],
                     ['PC.636', 'ACGGTGAGTGTC', 'YATGCTGCCTCCCGTAGGAGT',
                      'Fast', '20080116', 'Fasting_mouse_I.D._636']]


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# ----------------------------------------------------------------------------
# Copyright (c) 2013--, emperor development team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file LICENSE.md, distributed with this software.
# ----------------------------------------------------------------------------
from __future__ import division

from os import makedirs
from os.path import join, isdir

from qcli.option_parsing import parse_command_line_parameters, make_option
from skbio.util import flatten

from emperor.qiime_backports.filter import filter_mapping_file
from emperor.qiime_backports.parse import (parse_mapping_file,
                                           mapping_file_to_dict,
                                           parse_otu_table, QiimeParseError)
from emperor.qiime_backports.util import MetadataMap
from emperor.biplots import preprocess_otu_table
from emperor.sort import sort_comparison_filenames
from emperor.filter import keep_samples_from_pcoa_data
from emperor.util import (copy_support_files, preprocess_mapping_file,
                          preprocess_coords_file,
                          fill_mapping_field_from_mapping_file,
                          EmperorInputFilesError, guess_coordinates_files)
from emperor.format import (format_taxa_to_js, format_vectors_to_js,
                            format_emperor_html_footer_string,
                            format_mapping_file_to_js,
                            format_pcoa_to_js,
                            format_comparison_bars_to_js,
                            EmperorLogicError,
                            format_emperor_autograph)
from emperor._format_strings import EMPEROR_HEADER_HTML_STRING
from emperor.parse import parse_coords
from emperor import __version__


script_info = {}

script_info['brief_description'] = "Create three dimensional PCoA plots"
script_info['script_description'] = ("This script automates the creation of "
                                     "three-dimensional PCoA plots to be "
                                     "visualized with Emperor using Google "
                                     "Chrome.")
script_info['script_usage'] = [
    ("Plot PCoA data",

     "Visualize the a PCoA file colored using a corresponding mapping file: ",

     "%prog -i unweighted_unifrac_pc.txt -m Fasting_Map.txt -o emperor_output"
     ),

    ("Plot data generated from non-phylogenetic distance matrices",

     "Visualize a PCoA file where the data was computed with a "
     "non-phylogenetic distance metric (Euclidian, Bray-Curtis, etc.)",

     "%prog -i euclidian_pc.txt -m Fasting_Map.txt -o euclidian"
     ),

    ("Coloring by metadata mapping file",

     "Additionally, using the supplied mapping file and a specific category "
     "or any combination of the available categories. When using the -b "
     "option, the user can specify the coloring for multiple header names, "
     "where each header is separated by a comma. The user can also combine "
     "mapping headers and color by the combined headers that are created by "
     "inserting an '&&' between the input header names. Color by 'Treatment' "
     "and by the result of concatenating the 'DOB' category and the "
     "'Treatment' category: ",

     "%prog -i unweighted_unifrac_pc.txt -m Fasting_Map.txt -b "
     "'Treatment&&DOB,Treatment' -o emperor_colored_by"
     ),

    ("PCoA plot with an explicit axis",

     "Create a PCoA plot with an axis of the plot representing the 'DOB' of "
     "the samples. This option is useful when presenting a gradient from your "
     "metadata e. g. 'Time' or 'pH': ",

     "%prog -i unweighted_unifrac_pc.txt -m Fasting_Map.txt -a DOB -o "
     "pcoa_dob"
     ),

    ("PCoA plot with an explicit axis and using --missing_custom_axes_values",

     "Create a PCoA plot with an axis of the plot representing the 'DOB' of "
     "the samples and define the position over the gradient of those samples "
     "missing a numeric value; in this case we are going to plot the samples "
     "in the value 20060000. You can select for each explicit axis which "
     "value you want to use for the missing values: ",

     "%prog -i unweighted_unifrac_pc.txt -m Fasting_Map_modified.txt -a DOB "
     "-o pcoa_dob_with_missing_custom_axes_values -x 'DOB:20060000'"
     ),

    ("PCoA plot with an explicit axis and using --missing_custom_axes_values "
     "but setting different values based on another column",

     "Create a PCoA plot with an axis of the plot representing the 'DOB' of "
     "the samples and defining the position over the gradient of those "
     "samples missing a numeric value but using as reference another column "
     "of the mapping file. In this case we are going to plot the samples that "
     "are Control on the Treatment column on 20080220 and on 20080240 those "
     "that are Fast:",

     "%prog -i unweighted_unifrac_pc.txt -m Fasting_Map_modified.txt -a DOB "
     "-o pcoa_dob_with_missing_custom_axes_with_multiple_values -x "
     "'DOB:Treatment==Control=20080220' -x 'DOB:Treatment==Fast=20080240'"
     ),

    ("Jackknifed principal coordinates analysis plot",

     "Create a jackknifed PCoA plot (with confidence intervals for each "
     "sample) passing as the input a directory of coordinates files (where "
     "each file corresponds to a different OTU table) and use the standard "
     "deviation method to compute the dimensions of the ellipsoids "
     "surrounding each sample: ",

     "%prog -i unweighted_unifrac_pc -m Fasting_Map.txt -o "
     "jackknifed_pcoa -e sdev"
     ),

    ("Jackknifed PCoA plot with a master coordinates file",

     "Passing a master coordinates file (--master_pcoa) will display the "
     "ellipsoids centered by the samples in this file: ",

     "%prog -i unweighted_unifrac_pc -s "
     "unweighted_unifrac_pc/pcoa_unweighted_unifrac_rarefaction_110_5.txt -m "
     "Fasting_Map.txt -o jackknifed_with_master"
     ),

    ("BiPlots",

     "Create a distance biplot to visualize where samples and taxa are in "
     "PCoA space.  The cosine of the angle between taxanomic vectors "
     "approximates the correlation of those two taxa. The length of each of "
     "the taxonomic vectors approximates the variance of each taxa throughout "
     "all of the samples.  The direction of the taxanomic vector explains the "
     "variance of all of the samples along that direction. "
     "To see which taxa are the ten more prevalent in the different "
     "areas of the PCoA plot, you need to pass a summarized taxa file i.e. "
     "the output of summarize_taxa.py. Note that if the the '--taxa_fp' has "
     "fewer than 10 taxa, the script will default to use all.",

     "%prog -i unweighted_unifrac_pc.txt -m Fasting_Map.txt -t "
     "otu_table_L3.txt -o biplot"
     ),

    ("BiPlots with extra options",

     "To see which are the three most prevalent taxa and save the coordinates "
     "where these taxa are centered, you can use the -n (number of taxa to "
     "keep) and the --biplot_fp (output biplot file path) options.",

     "%prog -i unweighted_unifrac_pc.txt -m Fasting_Map.txt -t "
     "otu_table_L3.txt -o biplot_options -n 3 --biplot_fp biplot.txt"
     ),

    ("Drawing connecting lines between samples",

     "Tao draw lines betwen samples within a category use the '--add_vectors' "
     "option. For example to connect the lines by the 'Treatment' category.",

     "%prog -i unweighted_unifrac_pc.txt -m Fasting_Map.txt -o vectors "
     "--add_vectors Treatment"
     ),

    ("Drawing connecting lines between samples with an explicit axis",

     "To draw lines between samples within a category of the mapping file and "
     "have them sorted by a category that's explicitly represented in the 3D "
     "plot use the '--add_vectors' and the '-a' option.",

     "%prog -i unweighted_unifrac_pc.txt -m Fasting_Map.txt --add_vectors "
     "Treatment,DOB -a DOB -o sorted_by_DOB"
     ),

    ("Compare two coordinate files",

     "To draw replicates of the same samples like for a procustes plot.",

     "%prog -i compare -m Fasting_Map.txt --compare_plots -o comparison"
     )]

script_info['output_description'] = \
    ("This script creates an output directory with an HTML formated file "
     "named 'index.html' and a complementary folder named "
     "'emperor_required_resources'. Opening index.html with Google's Chrome "
     "web browser will display a three dimensional visualization of the "
     "processed PCoA data file and the corresponding metadata mapping file.")
script_info['required_options'] = [
    make_option('-i', '--input_coords', type="existing_path",
                help=("Depending on the plot to be generated, can be one of "
                      "the following: (1) Filepath of a coordinates file to "
                      "create a PCoA plot. (2) Directory path to a folder "
                      "containing coordinates files to create a jackknifed "
                      "PCoA plot. (3) Directory path to a folder containing "
                      "coordinates files to compare the coordinates there "
                      "contained when --compare_plots is enabled (useful for "
                      "procustes analysis plots). For directories: hidden "
                      "files, sub-directories, 'Icon?' and files suffixed by "
                      "'_procrustes_results.txt' will be ignored.")),
    make_option('-m', '--map_fp', type="existing_filepath",
                help='path to a metadata mapping file')
]

script_info['optional_options'] = [
    make_option('--number_of_axes', type=int,
                help=("Number of axes to be incorporated in the plot. Only 3 "
                      "will be displayed at any given time but this option "
                      "modifies how many axes you can use for your "
                      "visualization. Note that Emperor will only use the "
                      "axes that explain more than 0.5% (this will be shown "
                      "as 1% in the GUI)of the variability [default: "
                      "%default]"),
                default=10),
    make_option('-a', '--custom_axes', type='string',
                help=("Comma-separated list of metadata categories to use as "
                      "custom axes in the plot. For instance, if there is a "
                      "time category and you would like to see the samples "
                      "plotted on that axis instead of PC1, PC2, etc., you "
                      "would pass time as the value of this option.  Note: if "
                      "there is any non-numeric data in the metadata column, "
                      "an error will be presented [default: %default]"),
                default=None),
    make_option('--add_unique_columns', action="store_true",
                help=('Add to the output categories of the mapping file the '
                      'columns where all values are different. Note: if the '
                      'result of one of the concatenated fields in --color_by '
                      'is a column where all values are unique, the resulting '
                      'column will get removed as well [default: %default]'),
                default=False),
    make_option('--add_vectors', type='string', dest='add_vectors',
                help=('Comma-sparated category(ies) used to add connecting '
                      'lines (vectors) between samples. The first category '
                      'specifies the samples that will be connected by the '
                      'vectors, whilst the second category (optionally) '
                      'determines the order in which the samples will be '
                      'connected. [default: %default]'), default=[None, None]),
    make_option('-b', '--color_by', dest='color_by', type='string',
                help=('Comma-separated list of metadata categories (column '
                      'headers) to color by in the plots. The categories must '
                      'match the name of a column header in the mapping file '
                      'exactly. Multiple categories can be listed by comma '
                      'separating them without spaces. The user can also '
                      'combine columns in the mapping file by separating the '
                      'categories by "&&" without spaces. [default=color by '
                      'all categories except ones where all values are '
                      'different, to disable this behaviour pass '
                      '--add_unique_columns]'),
                default=None),
    make_option('--biplot_fp', type='new_filepath',
                help=('Output filepath that will contain the coordinates '
                      'where each taxonomic sphere is centered. [default: '
                      '%default]'),
                default=None),
    make_option('-c', '--compare_plots', dest='compare_plots',
                action='store_true', default=False,
                help=('Passing a directory with the -i (--input_coords) '
                      'option in combination with this flag results in a set '
                      'of bars connecting the replicated samples across all '
                      'the input files. [default=%default]')),
    make_option('-e', '--ellipsoid_method',
                help=('Used only when plotting ellipsoids for jackknifed beta '
                      'diversity (i.e. using a directory of coord files '
                      'instead of a single coord file). Valid values are '
                      '"IQR" (for inter-quartile ranges) and "sdev" (for '
                      'standard deviation). [default=%default]'),
                default='IQR', type='choice', choices=['IQR', 'sdev']),
    make_option('--ignore_missing_samples', action='store_true', default=False,
                help=('This will overpass the error raised when the '
                      'coordinates file contains samples that are not present '
                      'in the mapping file. Be aware that this is very '
                      'misleading as the PCoA is accounting for all the '
                      'samples and removing some samples could lead to '
                      'erroneous/skewed interpretations.')),
    make_option('-n', '--n_taxa_to_keep', default=10, type='int',
                help=('Number of taxonomic groups from the "--taxa_fp" file '
                      'to display. Passing "-1" will cause to display all the '
                      'taxonomic groups, this option is only used when '
                      'creating BiPlots. [default=%default]')),
    make_option('-s', '--master_pcoa',
                help=('Used only when the input is a directory of coordinate '
                      'files i. e. for jackknifed beta diversity plot or '
                      'for a coordinate comparison plot (procrustes '
                      'analysis). The coordinates in this file will be the '
                      'center of each ellipsoid in the case of a jackknifed '
                      'PCoA plot or the center where the connecting arrows '
                      'originate from for a comparison plot. [default: '
                      'arbitrarily selected file from the input directory for '
                      'a jackknifed plot or None for a comparison plot in '
                      'this case one file will be connected to the next one '
                      'and so on]'), default=None, type='existing_filepath'),
    make_option('-t', '--taxa_fp', default=None, type='existing_filepath',
                help=('Path to a summarized taxa file (i.e. the output of '
                      'summarize_taxa.py). This option is only used when '
                      'creating BiPlots. [default=%default]')),
    make_option('-x', '--missing_custom_axes_values',
                help=('Option to override the error shown when the catergory '
                      'used in \'--custom_axes\' has non-numeric values in '
                      'the mapping file. The basic format is '
                      'custom_axis:new_value. For example, if you want to '
                      'plot in time 0 all the samples that do not have a '
                      'numeric value in the column Time. you would pass -x '
                      '"Time:0". Additionally, you can pass this format '
                      'custom_axis:other_col==value_in_other_col=new_value, '
                      'with this format you can specify different values '
                      '(new_value) to use in the substitution based on other '
                      'column (other_column) value (value_in_other_column); '
                      'see example above. This option could be used in all '
                      'explicit axes.'), action='append', default=None),
    make_option('-o', '--output_dir', type="new_dirpath", default='emperor',
                help=('path to the output directory that will contain the '
                      'PCoA plot. [default: %default]')),
    make_option('--number_of_segments', type="int", default=8,
                help=('The number of segments to generate any spheres, this '
                      'includes the samples, the taxa (biplots), and the '
                      'confidence intervals (jackknifing). Higher values will '
                      'result in better quality but can make the plots less '
                      'responsive, also it will make the resulting SVG images '
                      'bigger. The value should be between 4 and 14. '
                      '[default: %default]')),
    make_option('--pct_variation_below_one', action="store_true",
                help=('Allow the percent variation explained by the axis to '
                      'be below one. The default behaivor is to multiply by '
                      '100 all values if PC1 is < 1.0 [default: %default]'),
                default=False),
]
script_info['version'] = __version__


def main():
    option_parser, opts, args = parse_command_line_parameters(**script_info)
    input_coords = opts.input_coords
    map_fp = opts.map_fp
    output_dir = opts.output_dir
    color_by_column_names = opts.color_by
    add_unique_columns = opts.add_unique_columns
    custom_axes = opts.custom_axes
    ignore_missing_samples = opts.ignore_missing_samples
    missing_custom_axes_values = opts.missing_custom_axes_values
    jackknifing_method = opts.ellipsoid_method
    master_pcoa = opts.master_pcoa
    taxa_fp = opts.taxa_fp
    n_taxa_to_keep = opts.n_taxa_to_keep
    biplot_fp = opts.biplot_fp
    add_vectors = opts.add_vectors
    verbose_output = opts.verbose
    number_of_axes = opts.number_of_axes
    compare_plots = opts.compare_plots
    number_of_segments = opts.number_of_segments
    pct_variation_below_one = opts.pct_variation_below_one

    # add some metadata to the output
    emperor_autograph = format_emperor_autograph(map_fp, input_coords, 'HTML')

    # verifying that the number of axes requested is greater than 3
    if number_of_axes < 3:
        option_parser.error(('You need to plot at least 3 axes.'))

    # verifying that the number of segments is between the desired range
    if not (4 <= number_of_segments <= 14):
        option_parser.error(('number_of_segments should be between 4 and 14.'))

    # append headernames that the script didn't find in the mapping file
    # according to different criteria to the following variables
    offending_fields = []
    non_numeric_categories = []

    serial_comparison = True

    # can't do averaged pcoa plots _and_ custom axes in the same plot
    if custom_axes is not None and isdir(input_coords):
        if custom_axes.count(',') > 0:
            option_parser.error(('Jackknifed plots are limited to one custom '
                                 'axis, currently trying to use: %s. Make '
                                 'sure you use only one.' % custom_axes))

    # make sure the flag is not misunderstood from the command line interface
    if not isdir(input_coords) and compare_plots:
        option_parser.error("Cannot use the '--compare_plots' flag unless the "
                            "input path is a directory.")

    # before creating any output, check correct parsing of the main input files
    try:
        mapping_data, header, comments = parse_mapping_file(open(map_fp, 'U'))

    except Exception:
        option_parser.error(("The metadata mapping file '%s' does not seem "
                             "to be formatted correctly, verify the "
                             "formatting is QIIME compliant by using "
                             "validate_mapping_file.py") % map_fp)
    else:
        # use this set variable to make presence/absensce checks faster
        lookup_header = set(header)
        mapping_ids = {row[0] for row in mapping_data}

    # dir means jackknifing or coordinate comparison type of processing
    if isdir(input_coords):
        offending_coords_fp = []
        coords_headers = []
        coords_data = []
        coords_eigenvalues = []
        coords_pct = []

        coord_fps = guess_coordinates_files(input_coords)

        # QIIME generates folders of transformed coordinates for the specific
        # purpose of connecting all coordinates to a set of origin coordinates.
        # The name of this file is suffixed as _transformed_reference.txt
        trans_suf = '_transformed_reference.txt'
        transformed = [f for f in coord_fps if f.endswith(trans_suf)]

        # this could happen and we rather avoid this problem
        if len(coord_fps) == 0:

            option_parser.error('Could not use any of the files in the input '
                                'directory.')

        # the master pcoa must be the first in the list of coordinates; however
        # if the visualization is not a jackknifed plot this gets ignored
        if master_pcoa and not compare_plots:
            if master_pcoa in coord_fps:  # remove it if duplicated
                coord_fps.remove(master_pcoa)
            coord_fps = [master_pcoa] + coord_fps  # prepend it to the list
        # passing a master file means that the comparison is not serial
        elif master_pcoa and compare_plots:
            serial_comparison = False

            # guarantee that the master is the first and is not repeated
            if master_pcoa in coord_fps:
                coord_fps.remove(master_pcoa)
                sorted_filenames = sort_comparison_filenames(coord_fps)
                coord_fps = [master_pcoa] + sorted_filenames

        elif master_pcoa is None and len(transformed):
            master_pcoa = transformed[0]
            serial_comparison = False

            # Note: the following steps are to guarantee consistency.
            # remove the master from the list and re-add it as a first element
            # the rest of the files must be sorted alphabetically so the result
            # will be: ['unifrac_transformed_reference.txt',
            # 'unifrac_transformed_q1.txt', 'unifrac_transformed_q2.txt'] etc
            coord_fps.remove(master_pcoa)
            coord_fps = [master_pcoa] + sort_comparison_filenames(coord_fps)

        for fp in coord_fps:
            try:
                parsed = parse_coords(open(fp, 'U'))
            except (ValueError, QiimeParseError):
                offending_coords_fp.append(fp)

                # do not add any of the data and move along
                continue
            else:
                # pack all the data correspondingly only if it was correctly
                # parsed
                coords_headers.append(parsed[0])
                coords_data.append(parsed[1])
                coords_eigenvalues.append(parsed[2])
                coords_pct.append(parsed[3])

        # in case there were files that couldn't be parsed
        if offending_coords_fp:
            errout = ', '.join(offending_coords_fp)
            option_parser.error(("The following file(s): '%s' could not be "
                                 "parsed properly. Make sure the input folder "
                                 "only contains coordinates files.") % errout)

        # check all files contain the same sample identifiers by flattening the
        # list of available sample ids and returning the sample ids that are
        # in one of the sets of sample ids but not in the globablly shared ids
        _coords_headers = set(flatten(coords_headers))
        _per_file_missing = [_coords_headers - set(e) for e in coords_headers]
        non_shared_ids = set(flatten(_per_file_missing))
        if non_shared_ids:
            errout = ', '.join(non_shared_ids)
            option_parser.error(("The following sample identifier(s): '%s' "
                                 "are not shared between all the files. The "
                                 "files used to make a jackknifed PCoA plot "
                                 "or coordinate comparison plot (procustes "
                                 "plot) must share all the same sample "
                                 "identifiers between each other.") % errout)

        # number of samples ids that are shared between coords and mapping
        # files
        sids_intersection = mapping_ids.intersection(_coords_headers)

        # sample ids that are not mapped but are in the coords
        sids_difference = _coords_headers.difference(mapping_ids)

        # used to perform different validations in the script, very similar for
        # the case where the input is not a directory
        number_intersected_sids = len(sids_intersection)
        required_number_of_sids = len(coords_headers[0])

    else:
        try:
            parsed = parse_coords(open(input_coords, 'U'))
        # this exception was noticed when there were letters in the coords file
        # other exeptions should be catched here; code will be updated then
        except (ValueError, QiimeParseError):
            option_parser.error(("The PCoA file '%s' does not seem to be a "
                                 "coordinates formatted file, verify by "
                                 "manually inspecting the contents.") %
                                input_coords)
        else:
            coords_headers = parsed[0]
            coords_data = parsed[1]
            coords_eigenvalues = parsed[2]
            coords_pct = parsed[3]

        # number of samples ids that are shared between coords and mapping
        # files
        sids_intersection = mapping_ids.intersection(coords_headers)

        # sample ids that are not mapped but are in the coords
        sids_difference = set(coords_headers).difference(mapping_ids)

        number_intersected_sids = len(sids_intersection)
        required_number_of_sids = len(coords_headers)

    if taxa_fp:
        try:
            # This should really use BIOM's Table.from_tsv
            # for summarized tables the "otu_ids" are really the "lineages"
            parsed = parse_otu_table(open(taxa_fp, 'U'), count_map_f=float,
                                     remove_empty_rows=True)
        except ValueError, e:
            option_parser.error(("There was a problem parsing the --taxa_fp: "
                                 "%s" % e.message))
        else:
            otu_sample_ids = parsed[0]
            lineages = parsed[1]
            otu_table = parsed[2]

        # make sure there are matching sample ids with the otu table
        if not sids_intersection.issuperset(otu_sample_ids):
            option_parser.error("The sample identifiers in the OTU table must "
                                "have at least one match with the data in the "
                                "mapping file and with the coordinates file. "
                                "Verify you are using input files that belong "
                                "to the same dataset.")
        if len(lineages) <= 1:
            option_parser.error("Contingency tables with one or fewer rows "
                                "are not supported, please try passing a "
                                "contingency table with more than one row.")
    else:
        # empty lists indicate that there was no taxa file passed in
        otu_sample_ids, lineages, otu_table = [], [], []

    # sample ids must be shared between files
    if number_intersected_sids <= 0:
        option_parser.error('None of your sample identifiers match between the'
                            ' mapping file and the coordinates file. Verify '
                            'you are using a coordinates file and a mapping '
                            'file that belong to the same dataset.')

    # the intersection of the sample ids in the coords and the sample ids in
    # the mapping file must at the very least include all ids in the coords
    # file Otherwise it isn't valid; unless --ignore_missing_samples is set
    # True
    if number_intersected_sids != required_number_of_sids:
        if ignore_missing_samples:
            # keep only the samples that are mapped in the mapping file
            coords_headers, coords_data = keep_samples_from_pcoa_data(
                coords_headers, coords_data, sids_intersection)
        else:
            message = ("The metadata mapping file has fewer sample "
                       "identifiers than the coordinates file. Verify you are "
                       "using a mapping file that contains at least all the "
                       "samples contained in the coordinates file(s). You can "
                       "force the script to ignore these samples by passing "
                       "the '--ignore_missing_samples' flag.")

            if verbose_output:
                missing_ids = ', '.join(sids_difference)
                message += ' Offending sample identifier(s): %s.' % missing_ids

            option_parser.error(message)

    # ignore samples that exist in the coords but not in the mapping file,
    # note: we're using sids_intersection so if --ignore_missing_samples is
    # enabled we account for unmapped coords, else the program will exit before
    # this point
    header, mapping_data = filter_mapping_file(mapping_data, header,
                                               sids_intersection,
                                               include_repeat_cols=True)

    # catch the errors that could occur when filling the mapping file values
    if missing_custom_axes_values:
        # the fact that this uses parse_metadata_state_descriptions makes the
        # following option '-x Category:7;PH:12' to work as well as the
        # script-interface-documented '-x Category:7 -x PH:12' option
        for val in missing_custom_axes_values:
            if ':' not in val:
                option_parser.error("Not valid missing value for custom "
                                    "axes: %s" % val)
        _mcav = ';'.join(missing_custom_axes_values)
        try:
            mapping_data = fill_mapping_field_from_mapping_file(mapping_data,
                                                                header, _mcav)
        except AssertionError, e:
            option_parser.error(e.message)
        except EmperorInputFilesError, e:
            option_parser.error(e.message)

    # check that all the required columns exist in the metadata mapping file
    if color_by_column_names:
        color_by_column_names = color_by_column_names.split(',')

        # check for all the mapping fields
        for col in color_by_column_names:
            # for concatenated columns check each individual field
            parts = col.split('&&')
            offending_fields.extend(p for p in parts if p not in lookup_header)
    else:
        # if the user didn't specify the header names display everything
        color_by_column_names = [None]

    # extract a list of the custom axes provided and each element is numeric
    if custom_axes:
        custom_axes = custom_axes.strip().strip("'").strip('"').split(',')

        # the MetadataMap object makes some checks easier
        map_object = MetadataMap(mapping_file_to_dict(mapping_data, header),
                                 [])
        for axis in custom_axes:
            # append the field to the error queue that it belongs to
            if axis not in lookup_header:
                offending_fields.append(axis)
                break
            # make sure this value is in the mapping file
            elif axis not in color_by_column_names:
                color_by_column_names.append(axis)
        # perform only if the for loop does not call break
        else:
            # make sure all these axes are numeric
            for axis in custom_axes:
                if not map_object.isNumericCategory(axis):
                    non_numeric_categories.append(axis)

    # make multiple checks for the add_vectors option
    if add_vectors != [None, None]:
        add_vectors = add_vectors.split(',')
        # check there are at the most two categories specified for this option
        if len(add_vectors) > 2:
            option_parser.error("The '--add_vectors' option can accept up to "
                                "two different fields from the mapping file; "
                                "currently trying to use %d (%s)." %
                                (len(add_vectors), ', '.join(add_vectors)))
        # make sure the field(s) exist
        for col in add_vectors:
            # concatenated fields are allowed now so check for each field
            if '&&' in col:
                for _col in col.split('&&'):
                    if _col not in lookup_header:
                        offending_fields.append(col)
                        break
                # only execute this block of code if all checked fields exist
                else:
                    # make sure that if it's going to be used for vector
                    # creation it gets used for coloring and map postprocessing
                    if col not in color_by_column_names:
                        color_by_column_names.append(col)
            # if it's a column without concatenations
            elif col not in lookup_header:
                offending_fields.append(col)
                break
            else:
                # check this vector value is in the color by category
                if col not in color_by_column_names:
                    color_by_column_names.append(col)
        # perform only if the for loop does not call break
        else:
            # check that the second category is all with numeric values
            if len(add_vectors) == 2:
                map_object = MetadataMap(mapping_file_to_dict(mapping_data,
                                                              header),
                                         [])
                # if it has non-numeric values add it to the list of offenders
                if not map_object.isNumericCategory(add_vectors[1]):
                    msg = add_vectors[1] + '(used in --add_vectors)'
                    non_numeric_categories.append(msg)
            else:
                add_vectors.append(None)

    # terminate the program for the cases where a mapping field was not found
    # or when a mapping field didn't meet the criteria of being numeric
    if offending_fields:
        option_parser.error("Invalid field(s) '%s'; the valid field(s) are: "
                            "'%s'" % (', '.join(offending_fields),
                                      ', '.join(header)))
    if non_numeric_categories:
        option_parser.error(("The following field(s): '%s' contain values "
                             "that are not numeric, hence not suitable for "
                             "'--custom_axes' nor for '--add_vectors'. Try "
                             "the '--missing_custom_axes_values' option to "
                             "fix these values." %
                             ', '.join(non_numeric_categories)))

    # process the coordinates file first, preventing the case where the custom
    # axes is not in the coloring categories i. e. in the --colory_by
    # categories
    preprocessed_coords = preprocess_coords_file(coords_headers, coords_data,
                                                 coords_eigenvalues,
                                                 coords_pct, header,
                                                 mapping_data, custom_axes,
                                                 jackknifing_method,
                                                 compare_plots,
                                                 pct_variation_below_one)
    coords_headers = preprocessed_coords[0]
    coords_data = preprocessed_coords[1]
    coords_eigenvalues = preprocessed_coords[2]
    coords_pct = preprocessed_coords[3]
    coords_low = preprocessed_coords[4]
    coords_high = preprocessed_coords[5]
    clones = preprocessed_coords[6]

    # process the otu table after processing the coordinates to get custom axes
    # (when available) or any other change that occurred to the coordinates
    preprocessed_otu_table = preprocess_otu_table(otu_sample_ids, otu_table,
                                                  lineages, coords_data,
                                                  coords_headers,
                                                  n_taxa_to_keep)
    otu_coords = preprocessed_otu_table[0]
    otu_table = preprocessed_otu_table[1]
    otu_lineages = preprocessed_otu_table[2]
    otu_prevalence = preprocessed_otu_table[3]
    lines = preprocessed_otu_table[4]

    # remove the columns in the mapping file that are not informative taking
    # into account the header names that were already authorized to be used
    # and take care of concatenating the fields for the && merged columns
    mapping_data, header = preprocess_mapping_file(mapping_data, header,
                                                   color_by_column_names,
                                                   not add_unique_columns,
                                                   clones=clones)

    # create the output directory before creating any other output
    if not isdir(opts.output_dir):
        makedirs(opts.output_dir)

    fp_out = open(join(output_dir, 'index.html'), 'w')
    fp_out.write(emperor_autograph+'\n')
    fp_out.write(EMPEROR_HEADER_HTML_STRING)

    # write the html file
    fp_out.write(format_mapping_file_to_js(mapping_data, header, header))

    # certain percents being explained cannot be displayed in the GUI
    try:
        fp_out.write(format_pcoa_to_js(coords_headers, coords_data,
                                       coords_pct, custom_axes, coords_low,
                                       coords_high,
                                       number_of_axes=number_of_axes,
                                       number_of_segments=number_of_segments))
    except EmperorLogicError, e:
        option_parser.error(e.message)

    fp_out.write(format_taxa_to_js(otu_coords, otu_lineages, otu_prevalence))
    fp_out.write(format_vectors_to_js(mapping_data, header, coords_data,
                                      coords_headers, add_vectors[0],
                                      add_vectors[1]))
    fp_out.write(format_comparison_bars_to_js(coords_data, coords_headers,
                                              clones, serial_comparison))
    has_taxa = taxa_fp is not None
    has_input_coords = isdir(input_coords) and not compare_plots
    has_add_vectors = add_vectors != [None, None]
    has_clones = clones > 0
    fp_out.write(format_emperor_html_footer_string(has_taxa, has_input_coords,
                                                   has_add_vectors,
                                                   has_clones))
    fp_out.close()
    copy_support_files(output_dir)

    # write the biplot coords in the output file if a path is passed
    if biplot_fp and taxa_fp:
        if biplot_fp.endswith('/') or isdir(biplot_fp):
            option_parser.error("Do not specify a path to a new (path ending "
                                "in a slash) or existing directory for "
                                "biplot_fp. The output file will be a "
                                "tab-delimited text file.")

        # make sure this file can be created
        try:
            fd = open(biplot_fp, 'w')
        except IOError:
            option_parser.error("There was a problem creating the file with "
                                "the coordinates for the biplots (%s)."
                                % biplot_fp)
        else:
            fd.writelines(lines)
            fd.close()


if __name__ == "__main__":
    main()



# ----------------------------------------------------------------------------
# Copyright (c) 2013--, emperor development team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file LICENSE.md, distributed with this software.
# ----------------------------------------------------------------------------
from __future__ import division

from unittest import TestCase, main

from numpy import array
from numpy.testing import assert_almost_equal

from emperor.sort import (sort_taxa_table_by_pcoa_coords,
                          sort_comparison_filenames)


class TopLevelTests(TestCase):
    def setUp(self):
        self.otu_headers = ['PC.636', 'PC.635', 'PC.356', 'PC.481', 'PC.354',
                            'PC.593', 'PC.355', 'PC.607', 'PC.634']

        self.otu_table = array(
            [[0.02739726, 0.04697987, 0.02, 0.04697987, 0.01, 0.02027027,
              0.01360544, 0.01342282, 0.02666667],
             [0.00684932, 0.02013423, 0.02, 0.00671141,  0., 0.00675676, 0.,
              0., 0.],
             [0.14383562, 0.27516779, 0.65333333, 0.52348993, 0.38926174,
              0.69594595, 0.28571429, 0.0738255, 0.19333333],
             [0., 0.02013423, 0.03333333, 0.01342282, 0., 0.0472973, 0.,
              0., 0.],
             [0.78767123, 0.45637584, 0.22, 0.39597315, 0.41610738, 0.20945946,
              0.70068027, 0.89932886, 0.77333333],
             [0., 0.02013423, 0.01333333, 0.00671141, 0.03355705, 0.00675676,
              0., 0., 0.],
             [0., 0., 0.01333333, 0., 0., 0., 0., 0., 0.],
             [0.03424658, 0.16107383, 0.02666667, 0.00671141, 0.14765101,
              0.01351351, 0., 0.01342282, 0.00666667]])

        self.coords = COORDS
        self.coords_header = ['PC.354', 'PC.356', 'PC.481', 'PC.593', 'PC.355',
                              'PC.607', 'PC.634', 'PC.636', 'PC.635']

        self.coord_fps = [
            'output_data/emperor/bray_curtis_pc_transformed_q1.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q10.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q11.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q12.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q13.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q14.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q15.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q16.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q17.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q18.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q19.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q2.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q20.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q21.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q22.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q23.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q24.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q25.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q26.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q27.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q28.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q29.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q3.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q4.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q5.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q6.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q7.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q8.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q9.txt']

        self.coord_fps_garbage = [
            'output_data/emperor/bray_qurtis_pc_transformed_q1.txt',
            'output_data/emperor/bray_111urtis_q_transformed_q10.txt',
            'output_data/emperor/aaaaaaa.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q12.txt',
            'output_data/emperor/qqq2223_curtis_qc_transformed_q13.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q14.txt',
            'output_data/emperor/bray_curtis_pc_transformed_reference.'
            'txtoutput_data/emperor/bray_curtis_pc_transformed_q15.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q16.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q17.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q18.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q19.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q2.txt',
            'output_data/emperor/boom.txt',
            'output_data/emperor/another_file with some characters '
            'and stuff .txt',
            'output_data/emperor/some_other_file_that_foo_wants_to_'
            'compare.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q23.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q24.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q25.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q26.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q27.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q28.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q29.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q3.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q4.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q5.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q6.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q7.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q8.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q9.txt']

    def test_sort_taxa_table_by_pcoa_coords(self):
        """Make sure OTU table and coordinates are sorted equally"""

        # case with shuffled inputs
        o_headers, o_otu_table = sort_taxa_table_by_pcoa_coords(
            self.coords_header, self.otu_table, self.otu_headers)

        self.assertEquals(o_headers, ['PC.354', 'PC.356', 'PC.481', 'PC.593',
                                      'PC.355', 'PC.607', 'PC.634', 'PC.636',
                                      'PC.635'])
        assert_almost_equal(o_otu_table, OTU_TABLE_A)

        # case with shuffled inputs and fewer samples
        o_headers, o_otu_table = sort_taxa_table_by_pcoa_coords(
            ['PC.354', 'PC.356', 'PC.635'], self.otu_table, self.otu_headers)
        self.assertEquals(o_headers, ['PC.354', 'PC.356', 'PC.635'])
        assert_almost_equal(o_otu_table, array(
            [[0.01, 0.02, 0.04697987],
             [0., 0.02, 0.02013423],
             [0.38926174, 0.65333333, 0.27516779],
             [0., 0.03333333, 0.02013423],
             [0.41610738, 0.22, 0.45637584],
             [0.03355705, 0.01333333, 0.02013423],
             [0., 0.01333333, 0.],
             [0.14765101, 0.02666667, 0.16107383]]))

    def test_sort_comparison_filenames_regular(self):
        """Check filenames are sorted correctly"""

        # check it correctly sorts the files according to the suffix
        out_sorted = sort_comparison_filenames(self.coord_fps)
        self.assertEquals(out_sorted, [
            'output_data/emperor/bray_curtis_pc_transformed_q1.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q2.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q3.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q4.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q5.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q6.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q7.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q8.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q9.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q10.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q11.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q12.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q13.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q14.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q15.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q16.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q17.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q18.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q19.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q20.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q21.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q22.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q23.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q24.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q25.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q26.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q27.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q28.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q29.txt'])

        # if files with garbage are passed in, the sorting should be still
        # consistent,putting the "garbaged" filenames at the beginning
        out_sorted = sort_comparison_filenames(self.coord_fps_garbage)
        self.assertEquals(out_sorted, [
            'output_data/emperor/aaaaaaa.txt',
            'output_data/emperor/boom.txt',
            'output_data/emperor/another_file with some characters and '
            'stuff .txt',
            'output_data/emperor/some_other_file_that_foo_wants_to_'
            'compare.txt',
            'output_data/emperor/bray_qurtis_pc_transformed_q1.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q2.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q3.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q4.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q5.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q6.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q7.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q8.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q9.txt',
            'output_data/emperor/bray_111urtis_q_transformed_q10.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q12.txt',
            'output_data/emperor/qqq2223_curtis_qc_transformed_q13.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q14.txt',
            'output_data/emperor/bray_curtis_pc_transformed_reference.'
            'txtoutput_data/emperor/bray_curtis_pc_transformed_q15.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q16.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q17.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q18.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q19.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q23.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q24.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q25.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q26.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q27.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q28.txt',
            'output_data/emperor/bray_curtis_pc_transformed_q29.txt'])

        # tricky case with extensions in things that are not the filename
        out_sorted = sort_comparison_filenames([
            'output_data_q1.txt/emperor/bray_curtis_pc_transformed_q9.txt',
            'output_data/emperorq11.txt/bray_curtis_pc_transformed_q2.txt',
            'output_data_q44.txt/emperor/bray_curtis_pc_transformed_q11.txt',
            'output_dataq-5.txt/emperor/bray_curtis_pc_transformed_q3.txt',
            'output_data_q511.txt/emperor/bray_curtis_pc_transformed_q1.txt'])
        self.assertEquals(out_sorted, [
            'output_data_q511.txt/emperor/bray_curtis_pc_transformed_q1.txt',
            'output_data/emperorq11.txt/bray_curtis_pc_transformed_q2.txt',
            'output_dataq-5.txt/emperor/bray_curtis_pc_transformed_q3.txt',
            'output_data_q1.txt/emperor/bray_curtis_pc_transformed_q9.txt',
            'output_data_q44.txt/emperor/bray_curtis_pc_transformed_q11.txt'])

        # make sure nothing happens when an empty list is passed
        self.assertEquals(sort_comparison_filenames([]), [])


COORDS = array(
    [[0.280399117569, -0.0060128286014, 0.0234854344148, -0.0468109474823,
      -0.146624450094, 0.00566979124596, -0.0354299634191, -0.255785794275,
      -4.84141986706e-09],
     [0.228820399536, -0.130142097093, -0.287149447883, 0.0864498846421,
      0.0442951919304, 0.20604260722, 0.0310003571386, 0.0719920436501,
      -4.84141986706e-09],
     [0.0422628480532, -0.0139681511889, 0.0635314615517, -0.346120552134,
      -0.127813807608, 0.0139350721063, 0.0300206887328, 0.140147849223,
      -4.84141986706e-09],
     [0.232872767451, 0.139788385269, 0.322871079774, 0.18334700682,
      0.0204661596818, 0.0540589147147, -0.0366250872041, 0.0998235721267,
      -4.84141986706e-09],
     [0.170517581885, -0.194113268955, -0.0308965283066, 0.0198086158783,
      0.155100062794, -0.279923941712, 0.0576092515759, 0.0242481862127,
      -4.84141986706e-09],
     [-0.0913299284215, 0.424147148265, -0.135627421345, -0.057519480907,
      0.151363490722, -0.0253935675552, 0.0517306152066, -0.038738217609,
      -4.84141986706e-09],
     [-0.349339228244, -0.120787589539, 0.115274502117, 0.0694953933826,
      -0.0253722182853, 0.067853201946, 0.244447634756, -0.0598827706386,
      -4.84141986706e-09],
     [-0.276542163845, -0.144964375408, 0.0666467344429, -0.0677109454288,
      0.176070269506, 0.072969390136, -0.229889463523, -0.0465989416581,
      -4.84141986706e-09],
     [-0.237661393984, 0.0460527772512, -0.138135814766, 0.159061025229,
      -0.247484698646, -0.115211468101, -0.112864033263, 0.0647940729676,
      -4.84141986706e-09]])

OTU_TABLE_A = array(
    [[0.01, 0.02, 0.04697987, 0.02027027, 0.01360544, 0.01342282, 0.02666667,
      0.02739726, 0.04697987],
     [0., 0.02, 0.00671141, 0.00675676, 0., 0., 0., 0.00684932, 0.02013423],
     [0.38926174, 0.65333333, 0.52348993, 0.69594595, 0.28571429, 0.0738255,
      0.19333333, 0.14383562, 0.27516779],
     [0., 0.03333333, 0.01342282, 0.0472973, 0., 0., 0., 0., 0.02013423],
     [0.41610738, 0.22, 0.39597315, 0.20945946, 0.70068027, 0.89932886,
      0.77333333, 0.78767123, 0.45637584],
     [0.03355705, 0.01333333, 0.00671141, 0.00675676, 0., 0., 0., 0.,
      0.02013423],
     [0., 0.01333333, 0., 0., 0., 0., 0., 0., 0.],
     [0.14765101, 0.02666667, 0.00671141, 0.01351351, 0., 0.01342282,
      0.00666667, 0.03424658, 0.16107383]])


if __name__ == "__main__":
    main()
